CREATE TABLE IDIOMES ( 
  Id NUMBER NOT NULL, 
  Nom VARCHAR2(20), 
  CONSTRAINT PK_IDIOMES PRIMARY KEY (Id)
) TABLESPACE DW;

COMMENT ON TABLE IDIOMES IS 'Taula d''Idiomes';

CREATE TABLE CENTRES(
  Id NUMBER NOT NULL,
  Tipus VARCHAR2(1) NOT NULL,
  Nom VARCHAR2(255) NOT NULL,
  DocTipus VARCHAR2(3) NOT NULL,
  DocNumero VARCHAR2(12) NOT NULL,
  Adrea VARCHAR2(255),
  CP VARCHAR2(7),
  Ciutat VARCHAR2(100),
  Provincia VARCHAR2(100),
  Pais VARCHAR2(50),
  DataBaixa DATE,
  IdIdioma NUMBER NOT NULL,
  CONSTRAINT PK_CENTRES PRIMARY KEY(Id),
  CONSTRAINT FK_CENTRES_IDIOMES FOREIGN KEY (IdIdioma) REFERENCES IDIOMES(Id),
  CONSTRAINT CK_CENTRES_DOCTIPUS CHECK (DocTipus IN ('PAS', 'NIE', 'CIF', 'NIF')),
  CONSTRAINT CK_CENTRES_TIPUS CHECK (Tipus IN ('T', 'C'))
) TABLESPACE DW;

COMMENT ON TABLE CENTRES IS 'Dades del Centre, tant si es Taller com Concessionari';
COMMENT ON COLUMN CENTRES.DocTipus IS 'Els valors possibles sn PAS - Passaport, NIE, CIF o NIF';
COMMENT ON COLUMN CENTRES.Tipus IS 'Els valors possibles sn T - Taller o C - Concessionari';
COMMENT ON COLUMN CENTRES.IdIdioma IS 'Idioma Principal del Centre';

CREATE TABLE TREBALLADORS(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(255) NOT NULL,
  Cognom1 VARCHAR2(255) NOT NULL,
  Cognom2 VARCHAR2(255),
  Rol VARCHAR2(2) NOT NULL,
  NomUsuari VARCHAR2(50) NOT NULL,
  Contrasenya VARCHAR2(255),
  DataBaixa DATE,
  IdCentre NUMBER NOT NULL,
  IdIdioma NUMBER NOT NULL,
  CONSTRAINT PK_TREBAL PRIMARY KEY(Id),
  CONSTRAINT FK_TREBAL_IDIOMES FOREIGN KEY (IdIdioma) REFERENCES IDIOMES(Id),
  CONSTRAINT FK_TREBAL_CENTRES FOREIGN KEY (IdCentre) REFERENCES CENTRES(Id),
  CONSTRAINT CK_TREBAL_ROL CHECK (Rol IN ('RE', 'EN','ME' ,'VE', 'AD'))
) TABLESPACE DW;

COMMENT ON TABLE TREBALLADORS IS 'Dades dels Treballadors dels Centres';
COMMENT ON COLUMN TREBALLADORS.Rol IS 'Els valors possibles sn RE - Responsable, EN - Encarregat, ME - Mecnic - VE - Venedor, AD - Administrador';
COMMENT ON COLUMN TREBALLADORS.IdIdioma IS 'Idioma Principal del Treballor';

CREATE TABLE CLIENTS(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(255) NOT NULL,
  Cognom1 VARCHAR2(255) NOT NULL,
  Cognom2 VARCHAR2(255),
  DataNaixement DATE,
  Sexe VARCHAR2(1) NOT NULL,
  DocTipus VARCHAR2(8) NOT NULL,
  DocNumero VARCHAR2(12) NOT NULL,
  Adrea VARCHAR2(255),
  CP VARCHAR2(7),
  Ciutat VARCHAR2(100),
  Provincia VARCHAR2(100),
  Pais VARCHAR2(50),
  DataBaixa DATE,
  IdIdioma NUMBER NOT NULL,
  CONSTRAINT PK_CLIENTS PRIMARY KEY(Id),
  CONSTRAINT FK_CLIENTS_IDIOMES FOREIGN KEY (IdIdioma) REFERENCES IDIOMES(Id),
  CONSTRAINT CK_CLIENTS_DOCTIPUS CHECK (DocTipus IN ('PASSPORT', 'NIE', 'CIF', 'NIF')),
  CONSTRAINT CK_CLIENTS_SEXE CHECK (Sexe IN ('H', 'D'))
) TABLESPACE DW;

COMMENT ON TABLE CLIENTS IS 'Dades de Clients';
COMMENT ON COLUMN CLIENTS.DocTipus IS 'Els valors possibles sn PASSPORT, NIE, CIF o NIF';
COMMENT ON COLUMN CLIENTS.Sexe IS 'Els valors possibles sn H - Home, D - Dona';
COMMENT ON COLUMN CLIENTS.IdIdioma IS 'Idioma Principal del Client';

CREATE TABLE CLIENTS_VEH(
  Id NUMBER NOT NULL,
  NomComplert VARCHAR2(500) NOT NULL,
  RangEdad VARCHAR2(25),
  NumVehicles NUMBER, 
  Sexe VARCHAR2(4) NOT NULL,
  DocTipus VARCHAR2(8) NOT NULL,
  DocNumero VARCHAR2(12) NOT NULL,
  Adrea VARCHAR2(255),
  CP VARCHAR2(7),
  Ciutat VARCHAR2(100),
  Provincia VARCHAR2(100),
  Pais VARCHAR2(50),
  DataBaixa DATE,
  IdIdioma NUMBER NOT NULL,
  CONSTRAINT PK_CLIENTS_VEH PRIMARY KEY(Id),
  CONSTRAINT FK_CLIENTS_VEH_IDIOMES FOREIGN KEY (IdIdioma) REFERENCES IDIOMES(Id),
  CONSTRAINT CK_CLIENTS_VEH_DOCTIPUS CHECK (DocTipus IN ('PASSPORT', 'NIE', 'CIF', 'NIF')),
  CONSTRAINT CK_CLIENTS_VEH_SEXE CHECK (Sexe IN ('Home', 'Dona'))
) TABLESPACE DW;

COMMENT ON TABLE CLIENTS_VEH IS 'Dades de Clients Amb Vehicles';
COMMENT ON COLUMN CLIENTS_VEH.DocTipus IS 'Els valors possibles sn PASSPORT, NIE, CIF o NIF';
COMMENT ON COLUMN CLIENTS_VEH.Sexe IS 'Els valors possibles sn Home, Dona';
COMMENT ON COLUMN CLIENTS_VEH.IdIdioma IS 'Idioma Principal del Client';

CREATE TABLE DISTRIBUIDORS(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(255) NOT NULL,
  NomContacte VARCHAR2(255),
  DocTipus VARCHAR2(8) NOT NULL,
  DocNumero VARCHAR2(12) NOT NULL,
  Adrea VARCHAR2(255),
  CP VARCHAR2(7),
  Ciutat VARCHAR2(100),
  Provincia VARCHAR2(100),
  Pais VARCHAR2(50),
  DataBaixa DATE,
  IdIdioma NUMBER NOT NULL,
  CONSTRAINT PK_DISTRI PRIMARY KEY(Id),
  CONSTRAINT FK_DISTRI_IDIOMES FOREIGN KEY (IdIdioma) REFERENCES IDIOMES(Id),
  CONSTRAINT CK_DISTRI_DOCTIPUS CHECK (DocTipus IN ('PASSPORT', 'NIE', 'CIF', 'NIF'))
) TABLESPACE DW;

COMMENT ON TABLE DISTRIBUIDORS IS 'Dades de Distribuidors';
COMMENT ON COLUMN DISTRIBUIDORS.DocTipus IS 'Els valors possibles sn PASSPORT, NIE, CIF o NIF';
COMMENT ON COLUMN DISTRIBUIDORS.IdIdioma IS 'Idioma Principal del Client';

CREATE TABLE MARQUES(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(255) NOT NULL,
  Pais VARCHAR2(50),
  CONSTRAINT PK_MARQUES PRIMARY KEY(Id)
) TABLESPACE DW;

COMMENT ON TABLE MARQUES IS 'Dades de Marques dels Vehicles';

CREATE TABLE MODELS(
  Id NUMBER NOT NULL,
  IdMarca NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  AnyModel NUMBER,
  CONSTRAINT PK_MODELS PRIMARY KEY(Id),
  CONSTRAINT FK_MODELS_MARQUES FOREIGN KEY (IdMarca) REFERENCES MARQUES(Id),
  CONSTRAINT CK_MODELS_ANY CHECK (AnyModel BETWEEN 1900 AND 2100)
) TABLESPACE DW;

COMMENT ON TABLE MODELS IS 'Dades de Models de Marques dels Vehicles';

CREATE TABLE COMBUSTIBLES(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  CONSTRAINT PK_COMBUS PRIMARY KEY(Id)
) TABLESPACE DW;

COMMENT ON TABLE COMBUSTIBLES IS 'Dades de Combustibles dels Vehicles';

CREATE TABLE POTENCIES(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  CONSTRAINT PK_POTENC PRIMARY KEY(Id)
) TABLESPACE DW;

COMMENT ON TABLE POTENCIES IS 'Dades de Potncies dels Vehicles';

CREATE TABLE CILINDRADES(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  CONSTRAINT PK_CILIND PRIMARY KEY(Id)
) TABLESPACE DW;

COMMENT ON TABLE CILINDRADES IS 'Dades de Cilindrades dels Vehicles';

CREATE TABLE VERSIO(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  AnyVersio NUMBER,
  Portes NUMBER,
  Cambi VARCHAR2(1) NOT NULL,
  CONSTRAINT PK_VERSIO PRIMARY KEY(Id),
  CONSTRAINT CK_VERSIO_ANY CHECK (AnyVersio BETWEEN 1900 AND 2100),
  CONSTRAINT CK_VERSIO_CAMBI CHECK (Cambi IN ('A', 'M'))
) TABLESPACE DW;

COMMENT ON TABLE VERSIO IS 'Dades de la Versi dels Vehicles';
COMMENT ON COLUMN VERSIO.Cambi IS 'Els valors possibles sn A - Automtic, M - Manual';  


CREATE TABLE DISTRIBUIDORS_MARQUES(
  Id NUMBER NOT NULL,
  IdDistribuidor NUMBER NOT NULL,
  IdMarca NUMBER  NOT NULL,
  CONSTRAINT PK_DISTRI_MARQUES PRIMARY KEY(Id),
  CONSTRAINT FK_DISTRI_MARQUES_DISTRI FOREIGN KEY (IdDistribuidor) REFERENCES DISTRIBUIDORS(Id),
  CONSTRAINT FK_DISTRI_MARQUES_MARQUES FOREIGN KEY (IdMarca) REFERENCES MARQUES(Id) 
) TABLESPACE DW;

COMMENT ON TABLE DISTRIBUIDORS_MARQUES IS 'Taula de relaci entre les marques i els seus distribuidors';

CREATE TABLE TIPUS_EXTRES(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  CONSTRAINT PK_TIPEXT PRIMARY KEY(Id)
) TABLESPACE DW;

COMMENT ON TABLE TIPUS_EXTRES IS 'Dades dels tipus d''Extres dels Vehicles';

CREATE TABLE EXTRES(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  IdTipusExtra NUMBER NOT NULL,
  Preu NUMBER NOT NULL,
  CONSTRAINT PK_EXTRES PRIMARY KEY(Id),
  CONSTRAINT FK_EXTRES_TIPEXT FOREIGN KEY (IdTipusExtra) REFERENCES TIPUS_EXTRES(Id)
) TABLESPACE DW;

COMMENT ON TABLE EXTRES IS 'Dades dels Extres dels Vehicles';

CREATE TABLE EXTRES_MODELS(
  Id NUMBER NOT NULL,
  IdModel NUMBER NOT NULL,
  IdExtra NUMBER NOT NULL,
  CONSTRAINT PK_EXTRES_MODELS PRIMARY KEY(Id),
  CONSTRAINT FK_EXTRES_MODELS_MODELS FOREIGN KEY (IdModel) REFERENCES MODELS(Id),
  CONSTRAINT FK_EXTRES_MODELS_EXTRES FOREIGN KEY (IdExtra) REFERENCES EXTRES(Id) 
) TABLESPACE DW;

COMMENT ON TABLE EXTRES_MODELS IS 'Taula de relaci entre els models de Vehicles i els seus possibles extres. Tots els vehicles d''aqull model poden disposar d''aquest extres';

CREATE TABLE VEHICLES(
  Id NUMBER NOT NULL,
  IdMarca NUMBER NOT NULL,
  IdModel NUMBER NOT NULL,
  IdVersio NUMBER NOT NULL,
  PreuVersio NUMBER NOT NULL,
  IdCilindrada NUMBER NOT NULL,
  IdPotencia NUMBER NOT NULL,
  IdCombustible NUMBER  NOT NULL,
  IdClient NUMBER,
  Bastidor VARCHAR2(20), 
  Matricula VARCHAR2(10),
  Km NUMBER DEFAULT 0 NOT NULL,
  DataRevisio DATE,
  DataBaixa DATE,
  AnyFabricacio NUMBER NOT NULL,
  CONSTRAINT PK_VEHICLES PRIMARY KEY(Id),
  CONSTRAINT FK_VEHICLES_MARQUES FOREIGN KEY (IdMarca) REFERENCES MARQUES(Id),
  CONSTRAINT FK_VEHICLES_MODELS FOREIGN KEY (IdModel) REFERENCES MODELS(Id),
  CONSTRAINT FK_VEHICLES_VERSIO FOREIGN KEY (IdVersio) REFERENCES VERSIO(Id),
  CONSTRAINT FK_VEHICLES_POTENCIES FOREIGN KEY (IdPotencia) REFERENCES POTENCIES(Id),
  CONSTRAINT FK_VEHICLES_CILINDRADES FOREIGN KEY (IdCilindrada) REFERENCES CILINDRADES(Id),
  CONSTRAINT FK_VEHICLES_COMBUSTIBLES FOREIGN KEY (IdCombustible) REFERENCES COMBUSTIBLES(Id),
  CONSTRAINT FK_VEHICLES_CLIENTS FOREIGN KEY (IdClient) REFERENCES CLIENTS(Id),
  CONSTRAINT CK_VEHICLES_ANY CHECK (AnyFabricacio BETWEEN 1900 AND 2100)
) TABLESPACE DW;

COMMENT ON TABLE VEHICLES IS 'Taula on guardem les dades de qualsevol vehicle que passa per algun centre, tant si es nouu, com segona ma o per fer reparaci o revisio';

CREATE TABLE VEHICLES_EXTRES(
  Id NUMBER NOT NULL,
  IdVehicle NUMBER NOT NULL,
  IdExtra NUMBER NOT NULL,
  CONSTRAINT PK_VEHICLES_EXTRES PRIMARY KEY(Id),
  CONSTRAINT FK_VEHICLES_EXTRES_VEHICLES FOREIGN KEY (IdVehicle) REFERENCES VEHICLES(Id),
  CONSTRAINT FK_VEHICLES_EXTRES_EXTRES FOREIGN KEY (IdExtra) REFERENCES EXTRES(Id) 
) TABLESPACE DW;

COMMENT ON TABLE VEHICLES_EXTRES IS 'Taula d''extres que te cada cotxe, b per que son de serie o per que els tenia en el moment de fer la compra o per configuraci final addicinal per la venta';

CREATE TABLE REPREV_ACCIONS(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  Preu NUMBER NOT NULL,
  CONSTRAINT PK_REPREV_ACCIONS PRIMARY KEY(Id)
) TABLESPACE DW;

COMMENT ON TABLE REPREV_ACCIONS IS 'Taula d''accions de realitzaci i solicitut en reparaci o revisi de vehicles';

CREATE TABLE REPREV(
  Id NUMBER NOT NULL,
  IdVehicle NUMBER NOT NULL,
  IdCentre NUMBER NOT NULL,
  IdClient NUMBER,
  Incidencia VARCHAR2(3) NOT NULL, 
  IdAccioSol NUMBER,
  DataEntrada DATE DEFAULT SYSDATE NOT NULL,
  DataSortida DATE,
  DataInici DATE,
  DataFi DATE,
  IdMecanicEnt NUMBER NOT NULL,
  IdMecanic NUMBER,
  IdMecanicSor NUMBER,
  Preu NUMBER,
  ComentariClient VARCHAR2(2000),
  ComentariMecanic VARCHAR2(2000),
  CONSTRAINT PK_REPREV PRIMARY KEY(Id),
  CONSTRAINT FK_REPREV_VEHICLES FOREIGN KEY (IdVehicle) REFERENCES VEHICLES(Id),
  CONSTRAINT FK_REPREV_ACCIO FOREIGN KEY (IdAccioSol) REFERENCES REPREV_ACCIONS(Id),
  CONSTRAINT FK_REPREV_CENTRES FOREIGN KEY (IdCentre) REFERENCES CENTRES(Id),
  CONSTRAINT FK_REPREV_CLIENTS FOREIGN KEY (IdClient) REFERENCES CLIENTS(Id),
  CONSTRAINT FK_REPREV_MECENT FOREIGN KEY (IdMecanicEnt) REFERENCES TREBALLADORS(Id),
  CONSTRAINT FK_REPREV_MECANIC FOREIGN KEY (IdMecanic) REFERENCES TREBALLADORS(Id),
  CONSTRAINT FK_REPREV_MECSOR FOREIGN KEY (IdMecanicSor) REFERENCES TREBALLADORS(Id),
  CONSTRAINT CK_VEHICLES_INCIDENCIA CHECK (Incidencia IN ('REV', 'REP'))
) TABLESPACE DW;

COMMENT ON TABLE REPREV IS 'Taula de gesti i histric de dades generals de reparacions i revisions de vehicles';
COMMENT ON COLUMN REPREV.Incidencia IS 'Els valors possibles sn REV - Revisi, REP - Reparaci';
COMMENT ON COLUMN REPREV.IdAccioSol IS 'Acci detallada per la que el client porta el vehicle';

CREATE TABLE REPREV_LINIES(
  Id NUMBER NOT NULL,
  IdRepRev NUMBER NOT NULL,
  IdAccio NUMBER NOT NULL,
  Preu NUMBER NOT NULL,
  Quantitat NUMBER NOT NULL,
  Comentari VARCHAR2 (255),
  CONSTRAINT PK_REPREV_LINIES PRIMARY KEY(Id),
  CONSTRAINT FK_REPREV_LINIES_REPREV FOREIGN KEY (IdRepRev) REFERENCES REPREV(Id),
  CONSTRAINT FK_REPREV_LINIES_RREVACC FOREIGN KEY (IdAccio) REFERENCES REPREV_ACCIONS(Id) 
) TABLESPACE DW;

COMMENT ON TABLE REPREV_LINIES IS 'Taula de les linees de detall de cada revisi o reparaci';

CREATE TABLE VENTES(
  Id NUMBER NOT NULL,
  IdVehicle NUMBER NOT NULL,
  IdClient NUMBER NOT NULL,
  IdCentre NUMBER NOT NULL,
  IdTreballador NUMBER NOT NULL,
  DataEncarrec DATE,
  Pagat NUMBER,
  DataVenta DATE,
  Preu NUMBER NOT NULL,
  Descompte NUMBER DEFAULT 0 NOT NULL,
  PreuFinal NUMBER NOT NULL,
  DataPrevEntrega DATE,
  DataEntrega DATE,
  Us VARCHAR2(1) NOT NULL,
  DataBaixa DATE,
  Comentari VARCHAR2(255),
  CONSTRAINT PK_VENTES PRIMARY KEY(Id),
  CONSTRAINT FK_VENTES_VEHICLES FOREIGN KEY (IdVehicle) REFERENCES VEHICLES(Id),
  CONSTRAINT FK_VENTES_CLIENTS FOREIGN KEY (IdClient) REFERENCES CLIENTS(Id),
  CONSTRAINT FK_VENTES_CENTRES FOREIGN KEY (IdCentre) REFERENCES CENTRES(Id),
  CONSTRAINT FK_VENTES_TREBALLADORS FOREIGN KEY (IdTreballador) REFERENCES TREBALLADORS(Id), 
  CONSTRAINT CK_VENTES_US CHECK (Us IN ('N', 'S'))
) TABLESPACE DW;

COMMENT ON TABLE VENTES IS 'Taula de ventes i histric de ventes de vehicles';
COMMENT ON COLUMN VENTES.Us IS 'Els valors possibles sn N - Nou, S - Segona ma';

CREATE TABLE COMPRES(
  Id NUMBER NOT NULL,
  IdCentre NUMBER NOT NULL,
  IdTreballador NUMBER NOT NULL,
  IdDistribuidor NUMBER,
  IdClient NUMBER,
  Us VARCHAR2(1) NOT NULL,
  DataCompra DATE DEFAULT SYSDATE NOT NULL,
  DataTancament DATE,
  IdAutoritzador NUMBER,
  Preu NUMBER NOT NULL,
  DataRecepcio DATE,
  IdReceptor NUMBER,
  DataBaixa DATE,
  Comentari VARCHAR2(255),
  CONSTRAINT PK_COMPRES PRIMARY KEY(Id),
  CONSTRAINT FK_COMPRES_CENTRES FOREIGN KEY (IdCentre) REFERENCES CENTRES(Id),
  CONSTRAINT FK_COMPRES_TREBALLADORS FOREIGN KEY (IdTreballador) REFERENCES TREBALLADORS(Id),   
  CONSTRAINT FK_COMPRES_DISTRIBUIDORS FOREIGN KEY (IdDistribuidor) REFERENCES DISTRIBUIDORS(Id),
  CONSTRAINT FK_COMPRES_CLIENTS FOREIGN KEY (IdClient) REFERENCES CLIENTS(Id),
  CONSTRAINT FK_COMPRES_AUTORITZADOR FOREIGN KEY (IdAutoritzador) REFERENCES TREBALLADORS(Id),  
  CONSTRAINT FK_COMPRES_RECEPTOR FOREIGN KEY (Idreceptor) REFERENCES TREBALLADORS(Id),
  CONSTRAINT CK_COMPRES_US CHECK (Us IN ('N', 'S'))
) TABLESPACE DW;

COMMENT ON TABLE COMPRES IS 'Taula de compres de vehicles';
COMMENT ON COLUMN COMPRES.Us IS 'Els valors possibles sn N - Nou, S - Segona ma';

CREATE TABLE COMPRES_LINIES(
  Id NUMBER NOT NULL,
  IdCompra NUMBER NOT NULL,
  IdVehicle NUMBER NOT NULL,
  Preu NUMBER NOT NULL,
  Comentari VARCHAR2(255),
  CONSTRAINT PK_COMPRES_LINIES PRIMARY KEY(Id),
  CONSTRAINT FK_COMPRES_LINIES_COMPRES FOREIGN KEY (IdCompra) REFERENCES COMPRES(Id),
  CONSTRAINT FK_COMPRES_LINIES_VEHICLES FOREIGN KEY (IdVehicle) REFERENCES VEHICLES(Id)
) TABLESPACE DW;

COMMENT ON TABLE COMPRES_LINIES IS 'Taula de detall de les compres de vehicles';

CREATE TABLE STOCK(
  Id NUMBER NOT NULL,
  IdVehicle NUMBER NOT NULL,  
  IdCentre NUMBER NOT NULL,
  DataEntrada DATE DEFAULT SYSDATE NOT NULL,
  DataEncarrec DATE, 
  DataSortida DATE,
  PreuVenta NUMBER NOT NULL,
  Us VARCHAR2(1) NOT NULL,
  CONSTRAINT PK_STOCK PRIMARY KEY(Id),
  CONSTRAINT FK_STOCK_VEHICLES FOREIGN KEY (IdVehicle) REFERENCES VEHICLES(Id), 
  CONSTRAINT FK_STOCK_CENTRES FOREIGN KEY (IdCentre) REFERENCES CENTRES(Id),
  CONSTRAINT CK_STOCK_US CHECK (Us IN ('N', 'S'))
) TABLESPACE DW;

COMMENT ON TABLE STOCK IS 'Taula de control de vehicles estocats';
COMMENT ON COLUMN STOCK.Us IS 'Els valors possibles sn N - Nou, S - Segona ma';


CREATE TABLE VEHICLES_VENUTS( 
IdVehicle NUMBER NOT NULL, 
IdMarca NUMBER NOT NULL, 
Marca VARCHAR2(100) NOT NULL,
IdModel NUMBER NOT NULL,
Model VARCHAR2(100) NOT NULL,
RangPreu VARCHAR2(8) NOT NULL,
RangEdadModel VARCHAR2(10) NOT NULL,
Us VARCHAR2(8) NOT NULL,
CONSTRAINT PK_VENUTS PRIMARY KEY(IdVehicle),
CONSTRAINT FK_VENUTS_VEHICLES FOREIGN KEY (IdVehicle) REFERENCES VEHICLES(Id), 
CONSTRAINT FK_VENUTS_MARQUES FOREIGN KEY (IdMarca) REFERENCES MARQUES(Id),
CONSTRAINT FK_VENUTS_MODELS FOREIGN KEY (IdModel) REFERENCES MODELS(Id)
) TABLESPACE DW;

CREATE TABLE LOGERROR(
  Id NUMBER NOT NULL,
  ErrorCode VARCHAR2(20) NOT NULL,
  Taula VARCHAR2(30),
  IdRegistre NUMBER,
  Descripcio VARCHAR2(255),
  DataError DATE DEFAULT SYSDATE NOT NULL, 
  Accio VARCHAR2(1) NOT NULL,
  IdTreballador NUMBER NOT NULL,
  CONSTRAINT PK_LOGERROR PRIMARY KEY(Id),
  CONSTRAINT FK_LOGERROR_TREBAL FOREIGN KEY (IdTreballador) REFERENCES TREBALLADORS(Id),
  CONSTRAINT CK_LOGERROR_ACCIO CHECK (Accio IN ('I', 'U', 'D', 'F', 'P', 'C', 'A'))
) TABLESPACE DW;

COMMENT ON TABLE LOGERROR IS 'Taula de logs';
COMMENT ON COLUMN LOGERROR.Accio IS 'Els valors possibles sn I - Insert, U - Update, D - Delete, F - Funci, P - Procs, C - Consulta, A - Altres';
