CREATE TABLE PERMISOS (
 Id NUMBER NOT NULL, 
 Accio VARCHAR2 (50) NOT NULL,
 Nivell NUMBER NOT NULL,
 CONSTRAINT PK_PERMISOS PRIMARY KEY (Id)
 );

CREATE TABLE IDIOMES ( 
  Id NUMBER NOT NULL, 
  Nom VARCHAR2(20), 
  CONSTRAINT PK_IDIOMES PRIMARY KEY (Id)
);

COMMENT ON TABLE IDIOMES IS 'Taula d''Idiomes';

CREATE TABLE MISSATGES(
  Id NUMBER NOT NULL,
  IdIdioma NUMBER NOT NULL,
  Text VARCHAR2(255),
  CONSTRAINT PK_MISSATGES PRIMARY KEY(Id, IdIdioma),
  CONSTRAINT FK_MISSATGES_IDIOMES FOREIGN KEY (IdIdioma) REFERENCES IDIOMES(Id)
);

COMMENT ON TABLE MISSATGES IS 'Taula de misstages mostrats per accions realitzades o errors';

CREATE TABLE CENTRES(
  Id NUMBER NOT NULL,
  Tipus VARCHAR2(1) NOT NULL,
  Nom VARCHAR2(255) NOT NULL,
  DocTipus VARCHAR2(3) NOT NULL,
  DocNumero VARCHAR2(12) NOT NULL,
  Adrea VARCHAR2(255),
  CP VARCHAR2(7),
  Ciutat VARCHAR2(100),
  Provincia VARCHAR2(100),
  Pais VARCHAR2(50),
  DataBaixa DATE,
  IdIdioma NUMBER NOT NULL,
  CONSTRAINT PK_CENTRES PRIMARY KEY(Id),
  CONSTRAINT FK_CENTRES_IDIOMES FOREIGN KEY (IdIdioma) REFERENCES IDIOMES(Id),
  CONSTRAINT CK_CENTRES_DOCTIPUS CHECK (DocTipus IN ('PAS', 'NIE', 'CIF', 'NIF')),
  CONSTRAINT CK_CENTRES_TIPUS CHECK (Tipus IN ('T', 'C'))
);

COMMENT ON TABLE CENTRES IS 'Dades del Centre, tant si es Taller com Concessionari';
COMMENT ON COLUMN CENTRES.DocTipus IS 'Els valors possibles sn PAS - Passaport, NIE, CIF o NIF';
COMMENT ON COLUMN CENTRES.Tipus IS 'Els valors possibles sn T - Taller o C - Concessionari';
COMMENT ON COLUMN CENTRES.IdIdioma IS 'Idioma Principal del Centre';

CREATE TABLE CENTRES_CONTACTE(
  Id NUMBER NOT NULL,
  IdCentre NUMBER NOT NULL,
  Tipus VARCHAR2(2) NOT NULL,
  Dada VARCHAR2(255) NOT NULL,
  CONSTRAINT PK_CENTRES_CONT PRIMARY KEY(Id),
  CONSTRAINT FK_CENTRES_CONT_CENTRES FOREIGN KEY (IdCentre) REFERENCES CENTRES(Id),
  CONSTRAINT CK_CENTRES_CONT_TIPUS CHECK (Tipus IN ('TP', 'TT', 'TM', 'EP', 'ET', 'FA'))
);
 
COMMENT ON TABLE CENTRES_CONTACTE IS 'Dades de contate dels Centres';
COMMENT ON COLUMN CENTRES_CONTACTE.Tipus IS 'Els valors possibles sn TP - Telfon Particular, TT - Telfon Treball, TM - Telfon Mbil, EP - Email Particular, ET - Email Treball, FA - Fax';

CREATE TABLE TREBALLADORS(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(255) NOT NULL,
  Cognom1 VARCHAR2(255) NOT NULL,
  Cognom2 VARCHAR2(255),
  Rol VARCHAR2(2) NOT NULL,
  NomUsuari VARCHAR2(50) NOT NULL,
  Contrasenya VARCHAR2(255),
  DataBaixa DATE,
  IdCentre NUMBER NOT NULL,
  IdIdioma NUMBER NOT NULL,
  CONSTRAINT PK_TREBAL PRIMARY KEY(Id),
  CONSTRAINT FK_TREBAL_IDIOMES FOREIGN KEY (IdIdioma) REFERENCES IDIOMES(Id),
  CONSTRAINT FK_TREBAL_CENTRES FOREIGN KEY (IdCentre) REFERENCES CENTRES(Id),
  CONSTRAINT CK_TREBAL_ROL CHECK (Rol IN ('RE', 'EN','ME' ,'VE', 'AD'))
);

COMMENT ON TABLE TREBALLADORS IS 'Dades dels Treballadors dels Centres';
COMMENT ON COLUMN TREBALLADORS.Rol IS 'Els valors possibles sn RE - Responsable, EN - Encarregat, ME - Mecnic - VE - Venedor, AD - Administrador';
COMMENT ON COLUMN TREBALLADORS.IdIdioma IS 'Idioma Principal del Treballor';

CREATE TABLE TREBALLADORS_CONTACTE(
  Id NUMBER NOT NULL,
  IdTreballador NUMBER NOT NULL,
  Tipus VARCHAR2(2) NOT NULL,
  Dada VARCHAR2(255) NOT NULL,
  CONSTRAINT PK_TREBAL_CONT PRIMARY KEY(Id),
  CONSTRAINT FK_TREBAL_CONT_TREBAL FOREIGN KEY (IdTreballador) REFERENCES TREBALLADORS(Id),
  CONSTRAINT CK_TREBAL_CONT_TIPUS CHECK (Tipus IN ('TP', 'TT', 'TM', 'EP', 'ET', 'FA'))
);
 
COMMENT ON TABLE TREBALLADORS_CONTACTE IS 'Dades de contate dels Treballadors';
COMMENT ON COLUMN TREBALLADORS_CONTACTE.Tipus IS 'Els valors possibles sn TP - Telfon Particular, TT - Telfon Treball, TM - Telfon Mbil, EP - Email Particular, ET - Email Treball, FA - Fax';  

CREATE TABLE CLIENTS(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(255) NOT NULL,
  Cognom1 VARCHAR2(255) NOT NULL,
  Cognom2 VARCHAR2(255),
  DataNaixement DATE,
  Sexe VARCHAR2(1) NOT NULL,
  DocTipus VARCHAR2(8) NOT NULL,
  DocNumero VARCHAR2(12) NOT NULL,
  Adrea VARCHAR2(255),
  CP VARCHAR2(7),
  Ciutat VARCHAR2(100),
  Provincia VARCHAR2(100),
  Pais VARCHAR2(50),
  DataBaixa DATE,
  IdIdioma NUMBER NOT NULL,
  CONSTRAINT PK_CLIENTS PRIMARY KEY(Id),
  CONSTRAINT FK_CLIENTS_IDIOMES FOREIGN KEY (IdIdioma) REFERENCES IDIOMES(Id),
  CONSTRAINT CK_CLIENTS_DOCTIPUS CHECK (DocTipus IN ('PASSPORT', 'NIE', 'CIF', 'NIF')),
  CONSTRAINT CK_CLIENTS_SEXE CHECK (Sexe IN ('H', 'D'))
);

COMMENT ON TABLE CLIENTS IS 'Dades de Clients';
COMMENT ON COLUMN CLIENTS.DocTipus IS 'Els valors possibles sn PASSPORT, NIE, CIF o NIF';
COMMENT ON COLUMN CLIENTS.Sexe IS 'Els valors possibles sn H - Home, D - Dona';
COMMENT ON COLUMN CLIENTS.IdIdioma IS 'Idioma Principal del Client';

CREATE TABLE CLIENTS_CONTACTE(
  Id NUMBER NOT NULL,
  IdClient NUMBER NOT NULL,
  Tipus VARCHAR2(2) NOT NULL,
  Dada VARCHAR2(255) NOT NULL,
  CONSTRAINT PK_CLIENTS_CONT PRIMARY KEY(Id),
  CONSTRAINT FK_CLIENTS_CONT_CLIENTS FOREIGN KEY (IdClient) REFERENCES CLIENTS(Id),
  CONSTRAINT CK_CLIENTS_CONT_TIPUS CHECK (Tipus IN ('TP', 'TT', 'TM', 'EP', 'ET', 'FA'))
);

COMMENT ON TABLE CLIENTS_CONTACTE IS 'Dades de contate dels Clients';
COMMENT ON COLUMN CLIENTS_CONTACTE.Tipus IS 'Els valors possibles sn TP - Telfon Particular, TT - Telfon Treball, TM - Telfon Mbil, EP - Email Particular, ET - Email Treball, FA - Fax';  


CREATE TABLE DISTRIBUIDORS(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(255) NOT NULL,
  NomContacte VARCHAR2(255),
  DocTipus VARCHAR2(8) NOT NULL,
  DocNumero VARCHAR2(12) NOT NULL,
  Adrea VARCHAR2(255),
  CP VARCHAR2(7),
  Ciutat VARCHAR2(100),
  Provincia VARCHAR2(100),
  Pais VARCHAR2(50),
  DataBaixa DATE,
  IdIdioma NUMBER NOT NULL,
  CONSTRAINT PK_DISTRI PRIMARY KEY(Id),
  CONSTRAINT FK_DISTRI_IDIOMES FOREIGN KEY (IdIdioma) REFERENCES IDIOMES(Id),
  CONSTRAINT CK_DISTRI_DOCTIPUS CHECK (DocTipus IN ('PASSPORT', 'NIE', 'CIF', 'NIF'))
);

COMMENT ON TABLE DISTRIBUIDORS IS 'Dades de Distribuidors';
COMMENT ON COLUMN DISTRIBUIDORS.DocTipus IS 'Els valors possibles sn PASSPORT, NIE, CIF o NIF';
COMMENT ON COLUMN DISTRIBUIDORS.IdIdioma IS 'Idioma Principal del Client';

CREATE TABLE DISTRIBUIDORS_CONTACTE(
  Id NUMBER NOT NULL,
  IdDistribuidor NUMBER NOT NULL,
  Tipus VARCHAR2(2) NOT NULL,
  Dada VARCHAR2(255) NOT NULL,
  CONSTRAINT PK_DISTRI_CONT PRIMARY KEY(Id),
  CONSTRAINT FK_DISTRI_CONT_DISTRI FOREIGN KEY (IdDistribuidor) REFERENCES DISTRIBUIDORS(Id),
  CONSTRAINT CK_DISTRI_CONT_TIPUS CHECK (Tipus IN ('TP', 'TT', 'TM', 'EP', 'ET', 'FA'))
);

COMMENT ON TABLE DISTRIBUIDORS_CONTACTE IS 'Dades de contate dels Distribuidors';
COMMENT ON COLUMN DISTRIBUIDORS_CONTACTE.Tipus IS 'Els valors possibles sn TP - Telfon Particular, TT - Telfon Treball, TM - Telfon Mbil, EP - Email Particular, ET - Email Treball, FA - Fax';  

CREATE TABLE MARQUES(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(255) NOT NULL,
  Pais VARCHAR2(50),
  CONSTRAINT PK_MARQUES PRIMARY KEY(Id)
);

COMMENT ON TABLE MARQUES IS 'Dades de Marques dels Vehicles';

CREATE TABLE MODELS(
  Id NUMBER NOT NULL,
  IdMarca NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  AnyModel NUMBER,
  CONSTRAINT PK_MODELS PRIMARY KEY(Id),
  CONSTRAINT FK_MODELS_MARQUES FOREIGN KEY (IdMarca) REFERENCES MARQUES(Id),
  CONSTRAINT CK_MODELS_ANY CHECK (AnyModel BETWEEN 1900 AND 2100)
);

COMMENT ON TABLE MODELS IS 'Dades de Models de Marques dels Vehicles';

CREATE TABLE MODELS_IMATGES(
  Id NUMBER NOT NULL,
  IdModel NUMBER NOT NULL,
  ImatgeG VARCHAR2(255) NOT NULL,
  ImatgeP VARCHAR2(255), 
  CONSTRAINT PK_MODELS_IMATGES PRIMARY KEY(Id),
  CONSTRAINT FK_MODELS_IMATGES_MODELS FOREIGN KEY (IdModel) REFERENCES MODELS(Id) 
);

COMMENT ON TABLE MODELS_IMATGES IS 'Taula de fotografies dels Models de Vehicles';

CREATE TABLE MODELS_IMATGES_IDIOMA(
  Id NUMBER NOT NULL,
  IdModelImatge NUMBER NOT NULL,
  IdIdioma NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  Descripcio VARCHAR2(255), 
  CONSTRAINT PK_MODIMAIDI PRIMARY KEY(Id),
  CONSTRAINT FK_MODIMAIDI_IDIOMA FOREIGN KEY (IdIdioma) REFERENCES IDIOMES(Id),
  CONSTRAINT FK_MODIMAIDI_MODELS_IMATGES FOREIGN KEY (IdModelImatge) REFERENCES MODELS_IMATGES(Id) 
);

COMMENT ON TABLE MODELS_IMATGES_IDIOMA IS 'Dades de nom i Descripci de les imatges dels Models Traduits';

CREATE TABLE COMBUSTIBLES(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  CONSTRAINT PK_COMBUS PRIMARY KEY(Id)
);

COMMENT ON TABLE COMBUSTIBLES IS 'Dades de Combustibles dels Vehicles';

CREATE TABLE COMBUSTIBLES_IDIOMA(
  Id NUMBER NOT NULL,
  IdCombustible NUMBER NOT NULL,
  IdIdioma NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  Descripcio VARCHAR2(255), 
  CONSTRAINT PK_COMBUS_IDIOMA PRIMARY KEY(Id),
  CONSTRAINT FK_COMBUS_IDIOMA_IDIOMA FOREIGN KEY (IdIdioma) REFERENCES IDIOMES(Id),
  CONSTRAINT FK_COMBUS_IDIOMA_COMBUS FOREIGN KEY (IdCombustible) REFERENCES COMBUSTIBLES(Id) 
);

COMMENT ON TABLE COMBUSTIBLES_IDIOMA IS 'Dades de Combustibles dels Vehicles Traduits';

CREATE TABLE POTENCIES(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  CONSTRAINT PK_POTENC PRIMARY KEY(Id)
);

COMMENT ON TABLE POTENCIES IS 'Dades de Potncies dels Vehicles';

CREATE TABLE POTENCIES_IDIOMA(
  Id NUMBER NOT NULL,
  IdPotencia NUMBER NOT NULL,
  IdIdioma NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  Descripcio VARCHAR2(255), 
  CONSTRAINT PK_POTENC_IDIOMA PRIMARY KEY(Id),
  CONSTRAINT FK_POTENC_IDIOMA_IDIOMA FOREIGN KEY (IdIdioma) REFERENCES IDIOMES(Id),
  CONSTRAINT FK_POTENC_IDIOMA_POTENC FOREIGN KEY (IdPotencia) REFERENCES POTENCIES(Id) 
);

COMMENT ON TABLE POTENCIES_IDIOMA IS 'Dades de Potncies dels Vehicles Traduits';

CREATE TABLE CILINDRADES(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  CONSTRAINT PK_CILIND PRIMARY KEY(Id)
);

COMMENT ON TABLE CILINDRADES IS 'Dades de Cilindrades dels Vehicles';

CREATE TABLE CILINDRADES_IDIOMA(
  Id NUMBER NOT NULL,
  IdCilindrada NUMBER NOT NULL,
  IdIdioma NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  Descripcio VARCHAR2(255), 
  CONSTRAINT PK_CILIND_IDIOMA PRIMARY KEY(Id),
  CONSTRAINT FK_CILIND_IDIOMA_IDIOMA FOREIGN KEY (IdIdioma) REFERENCES IDIOMES(Id),
  CONSTRAINT FK_CILIND_IDIOMA_CILIND FOREIGN KEY (IdCilindrada) REFERENCES CILINDRADES(Id) 
);

COMMENT ON TABLE CILINDRADES_IDIOMA IS 'Dades de Cilindrades dels Vehicles Traduits';

CREATE TABLE VERSIO(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  AnyVersio NUMBER,
  Portes NUMBER,
  Cambi VARCHAR2(1) NOT NULL,
  CONSTRAINT PK_VERSIO PRIMARY KEY(Id),
  CONSTRAINT CK_VERSIO_ANY CHECK (AnyVersio BETWEEN 1900 AND 2100),
  CONSTRAINT CK_VERSIO_CAMBI CHECK (Cambi IN ('A', 'M'))
);

COMMENT ON TABLE VERSIO IS 'Dades de la Versi dels Vehicles';
COMMENT ON COLUMN VERSIO.Cambi IS 'Els valors possibles sn A - Automtic, M - Manual';  


CREATE TABLE VERSIO_IDIOMA(
  Id NUMBER NOT NULL,
  IdVersio NUMBER NOT NULL,
  IdIdioma NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  Descripcio VARCHAR2(255), 
  CONSTRAINT PK_VERSIO_IDIOMA PRIMARY KEY(Id),
  CONSTRAINT FK_VERSIO_IDIOMA_IDIOMA FOREIGN KEY (IdIdioma) REFERENCES IDIOMES(Id),
  CONSTRAINT FK_VERSIO_IDIOMA_VERSIO FOREIGN KEY (IdVersio) REFERENCES VERSIO(Id) 
);

COMMENT ON TABLE VERSIO_IDIOMA IS 'Dades de Versi dels Vehicles Traduits';

CREATE TABLE DISTRIBUIDORS_MARQUES(
  Id NUMBER NOT NULL,
  IdDistribuidor NUMBER NOT NULL,
  IdMarca NUMBER  NOT NULL,
  CONSTRAINT PK_DISTRI_MARQUES PRIMARY KEY(Id),
  CONSTRAINT FK_DISTRI_MARQUES_DISTRI FOREIGN KEY (IdDistribuidor) REFERENCES DISTRIBUIDORS(Id),
  CONSTRAINT FK_DISTRI_MARQUES_MARQUES FOREIGN KEY (IdMarca) REFERENCES MARQUES(Id) 
);

COMMENT ON TABLE DISTRIBUIDORS_MARQUES IS 'Taula de relaci entre les marques i els seus distribuidors';

CREATE TABLE TIPUS_EXTRES(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  CONSTRAINT PK_TIPEXT PRIMARY KEY(Id)
);

COMMENT ON TABLE TIPUS_EXTRES IS 'Dades dels tipus d''Extres dels Vehicles';

CREATE TABLE TIPUS_EXTRES_IDIOMA(
  Id NUMBER NOT NULL,
  IdTipusExtra NUMBER NOT NULL,
  IdIdioma NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  Descripcio VARCHAR2(255), 
  CONSTRAINT PK_TIPEXT_IDIOMA PRIMARY KEY(Id),
  CONSTRAINT FK_TIPEXT_IDIOMA_IDIOMA FOREIGN KEY (IdIdioma) REFERENCES IDIOMES(Id),
  CONSTRAINT FK_TIPEXT_IDIOMA_TIPEXT FOREIGN KEY (IdTipusExtra) REFERENCES TIPUS_EXTRES(Id) 
);

COMMENT ON TABLE TIPUS_EXTRES_IDIOMA IS 'Dades Tipus d''Extres dels Vehicles Traduits';

CREATE TABLE EXTRES(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  IdTipusExtra NUMBER NOT NULL,
  Preu NUMBER NOT NULL,
  CONSTRAINT PK_EXTRES PRIMARY KEY(Id),
  CONSTRAINT FK_EXTRES_TIPEXT FOREIGN KEY (IdTipusExtra) REFERENCES TIPUS_EXTRES(Id)
);

COMMENT ON TABLE EXTRES IS 'Dades dels Extres dels Vehicles';

CREATE TABLE EXTRES_IDIOMA(
  Id NUMBER NOT NULL,
  IdExtra NUMBER NOT NULL,
  IdIdioma NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  Descripcio VARCHAR2(255), 
  CONSTRAINT PK_EXTRES_IDIOMA PRIMARY KEY(Id),
  CONSTRAINT FK_EXTRES_IDIOMA_IDIOMA FOREIGN KEY (IdIdioma) REFERENCES IDIOMES(Id),
  CONSTRAINT FK_EXTRES_IDIOMA_EXTRES FOREIGN KEY (IdExtra) REFERENCES EXTRES(Id) 
);

COMMENT ON TABLE EXTRES_IDIOMA IS 'Dades dels Extres dels Vehicles Traduits';

CREATE TABLE CONFIG1(
  Id NUMBER NOT NULL,
  IdModel NUMBER NOT NULL,
  IdCombustible NUMBER  NOT NULL,
  CONSTRAINT PK_CONFIG1 PRIMARY KEY(Id),
  CONSTRAINT FK_CONFIG1_COMBUS FOREIGN KEY (IdCombustible) REFERENCES COMBUSTIBLES(Id),
  CONSTRAINT FK_CONFIG1_MODELS FOREIGN KEY (IdModel) REFERENCES MODELS(Id) 
);

COMMENT ON TABLE CONFIG1 IS 'Taula de relaci entre les Models de Vehicles i els combustibles que accepta';

CREATE TABLE CONFIG2(
  Id NUMBER NOT NULL,
  IdConfig1 NUMBER NOT NULL,
  IdPotencia NUMBER NOT NULL,
  CONSTRAINT PK_CONFIG2 PRIMARY KEY(Id),
  CONSTRAINT FK_CONFIG2_CONFIG1 FOREIGN KEY (IdConfig1) REFERENCES CONFIG1(Id),
  CONSTRAINT FK_CONFIG2_POTENCIES FOREIGN KEY (IdPotencia) REFERENCES POTENCIES(Id) 
);

COMMENT ON TABLE CONFIG2 IS 'Taula de relaci entre la Configuraci1 de Vehicles i les potncies que accepta';

CREATE TABLE CONFIG3(
  Id NUMBER NOT NULL,
  IdConfig2 NUMBER NOT NULL,
  IdCilindrada NUMBER NOT NULL,
  CONSTRAINT PK_CONFIG3 PRIMARY KEY(Id),
  CONSTRAINT FK_CONFIG3_CONFIG2 FOREIGN KEY (IdConfig2) REFERENCES CONFIG2(Id),
  CONSTRAINT FK_CONFIG3_CILINDRADES FOREIGN KEY (IdCilindrada) REFERENCES CILINDRADES(Id) 
);

COMMENT ON TABLE CONFIG3 IS 'Taula de relaci entre la Configuraci2 de Vehicles i les Cilindrades que accepta';

CREATE TABLE CONFIGFIN(
  Id NUMBER NOT NULL,
  IdConfig3 NUMBER NOT NULL,
  IdVersio NUMBER NOT NULL,
  Preu NUMBER DEFAULT 0 NOT NULL, 
  CONSTRAINT PK_CONFIGFIN PRIMARY KEY(Id),
  CONSTRAINT FK_CONFIGFIN_CONFIG3 FOREIGN KEY (IdConfig3) REFERENCES CONFIG3(Id),
  CONSTRAINT FK_CONFIGFIN_VERSIO FOREIGN KEY (IdVersio) REFERENCES VERSIO(Id) 
);

COMMENT ON TABLE CONFIGFIN IS 'Taula de relaci entre la Configuraci3 de Vehicles i les Versions de vehicles existents';

CREATE TABLE CONFIGFIN_IMATGES(
  Id NUMBER NOT NULL,
  IdConfigFin NUMBER NOT NULL,
  ImatgeG VARCHAR2(255) NOT NULL,
  ImatgeP VARCHAR2(255), 
  CONSTRAINT PK_CONFIGFIN_IMATGES PRIMARY KEY(Id),
  CONSTRAINT FK_CONFIGFIN_IMATGES_CONFIGFIN FOREIGN KEY (IdConfigFin) REFERENCES CONFIGFIN(Id) 
);

COMMENT ON TABLE CONFIGFIN_IMATGES IS 'Taula de fotografies de les Configuracions Finals dels Vehicles';

CREATE TABLE CONFIGFIN_IMATGES_IDIOMA(
  Id NUMBER NOT NULL,
  IdConfigFinImatge NUMBER NOT NULL,
  IdIdioma NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  Descripcio VARCHAR2(255), 
  CONSTRAINT PK_CONIMAIDI PRIMARY KEY(Id),
  CONSTRAINT FK_CONIMAIDI_IDIOMA FOREIGN KEY (IdIdioma) REFERENCES IDIOMES(Id),
  CONSTRAINT FK_CONIMAIDI_CONFIGFIN_IMATGES FOREIGN KEY (IdConfigFinImatge) REFERENCES CONFIGFIN_IMATGES(Id) 
);

COMMENT ON TABLE CONFIGFIN_IMATGES_IDIOMA IS 'Dades de nom i Descripci de les imatges de les Configuracions finals Traduides';

CREATE TABLE EXTRES_MODELS(
  Id NUMBER NOT NULL,
  IdModel NUMBER NOT NULL,
  IdExtra NUMBER NOT NULL,
  CONSTRAINT PK_EXTRES_MODELS PRIMARY KEY(Id),
  CONSTRAINT FK_EXTRES_MODELS_MODELS FOREIGN KEY (IdModel) REFERENCES MODELS(Id),
  CONSTRAINT FK_EXTRES_MODELS_EXTRES FOREIGN KEY (IdExtra) REFERENCES EXTRES(Id) 
);

COMMENT ON TABLE EXTRES_MODELS IS 'Taula de relaci entre els models de Vehicles i els seus possibles extres. Tots els vehicles d''aqull model poden disposar d''aquest extres';

CREATE TABLE EXTRES_CONFIGFIN(
  Id NUMBER NOT NULL,
  IdConfigFin NUMBER NOT NULL,
  IdExtra NUMBER NOT NULL,
  CONSTRAINT PK_EXTRES_CONFIGFIN PRIMARY KEY(Id),
  CONSTRAINT FK_EXTRES_CONFIGFIN_CONFIGFIN FOREIGN KEY (IdConfigFin) REFERENCES CONFIGFIN(Id),
  CONSTRAINT FK_EXTRES_CONFIGFIN_EXTRES FOREIGN KEY (IdExtra) REFERENCES EXTRES(Id) 
);

COMMENT ON TABLE EXTRES_CONFIGFIN IS 'Taula de relaci entre els Vehicles Configurats fins la versi i els possibles extres que pot tenir. No pot acceptar un extre que ja el te el seu model';

CREATE TABLE VEHICLES(
  Id NUMBER NOT NULL,
  IdConfigFin NUMBER NOT NULL,
  IdClient NUMBER,
  Bastidor VARCHAR2(20), 
  Matricula VARCHAR2(10),
  Km NUMBER DEFAULT 0 NOT NULL,
  DataRevisio DATE,
  DataBaixa DATE,
  AnyFabricacio NUMBER NOT NULL,
  CONSTRAINT PK_VEHICLES PRIMARY KEY(Id),
  CONSTRAINT FK_VEHICLES_CONFIGFIN FOREIGN KEY (IdConfigFin) REFERENCES CONFIGFIN(Id),
  CONSTRAINT FK_VEHICLES_CLIENTS FOREIGN KEY (IdClient) REFERENCES CLIENTS(Id),
  CONSTRAINT CK_VEHICLES_ANY CHECK (AnyFabricacio BETWEEN 1900 AND 2100)
);

COMMENT ON TABLE VEHICLES IS 'Taula on guardem les dades de qualsevol vehicle que passa per algun centre, tant si es nouu, com segona ma o per fer reparaci o revisio';

CREATE TABLE VEHICLES_IMATGES(
  Id NUMBER NOT NULL,
  IdVehicle NUMBER NOT NULL,
  ImatgeG VARCHAR2(255) NOT NULL,
  ImatgeP VARCHAR2(255), 
  CONSTRAINT PK_VEHICLES_IMATGES PRIMARY KEY(Id),
  CONSTRAINT FK_VEHICLES_IMATGES_VEHICLES FOREIGN KEY (IdVehicle) REFERENCES VEHICLES(Id) 
);

COMMENT ON TABLE VEHICLES_IMATGES IS 'Taula de fotografies dels Vehicles';

CREATE TABLE VEHICLES_IMATGES_IDIOMA(
  Id NUMBER NOT NULL,
  IdVehicleImatge NUMBER NOT NULL,
  IdIdioma NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  Descripcio VARCHAR2(255), 
  CONSTRAINT PK_VEHIMAIDI PRIMARY KEY(Id),
  CONSTRAINT FK_VEHIMAIDI_IDIOMA FOREIGN KEY (IdIdioma) REFERENCES IDIOMES(Id),
  CONSTRAINT FK_VEHIMAIDI_VEHICLES_IMATGES FOREIGN KEY (IdVehicleImatge) REFERENCES VEHICLES_IMATGES(Id) 
);

COMMENT ON TABLE VEHICLES_IMATGES_IDIOMA IS 'Dades de nom i Descripci de les imatges dels Vehicles Traduides';

CREATE TABLE VEHICLES_EXTRES(
  Id NUMBER NOT NULL,
  IdVehicle NUMBER NOT NULL,
  IdExtra NUMBER NOT NULL,
  CONSTRAINT PK_VEHICLES_EXTRES PRIMARY KEY(Id),
  CONSTRAINT FK_VEHICLES_EXTRES_VEHICLES FOREIGN KEY (IdVehicle) REFERENCES VEHICLES(Id),
  CONSTRAINT FK_VEHICLES_EXTRES_EXTRES FOREIGN KEY (IdExtra) REFERENCES EXTRES(Id) 
);

COMMENT ON TABLE VEHICLES_EXTRES IS 'Taula d''extres que te cada cotxe, b per que son de serie o per que els tenia en el moment de fer la compra o per configuraci final addicinal per la venta';

CREATE TABLE REPREV_ACCIONS(
  Id NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  Preu NUMBER NOT NULL,
  CONSTRAINT PK_REPREV_ACCIONS PRIMARY KEY(Id)
);

COMMENT ON TABLE REPREV_ACCIONS IS 'Taula d''accions de realitzaci i solicitut en reparaci o revisi de vehicles';

CREATE TABLE REPREV_ACCIONS_IDIOMA(
  Id NUMBER NOT NULL,
  IdRepRevAccio NUMBER NOT NULL,
  IdIdioma NUMBER NOT NULL,
  Nom VARCHAR2(100) NOT NULL,
  Descripcio VARCHAR2(255), 
  CONSTRAINT PK_RREVACC_IDIOMA PRIMARY KEY(Id),
  CONSTRAINT FK_RREVACC_IDIOMA_IDIOMA FOREIGN KEY (IdIdioma) REFERENCES IDIOMES(Id),
  CONSTRAINT FK_RREVACC_IDIOMA_RREVCC FOREIGN KEY (IdRepRevAccio) REFERENCES REPREV_ACCIONS(Id) 
);

COMMENT ON TABLE REPREV_ACCIONS_IDIOMA IS 'Dades d''accions de reparaci i revisi dels Vehicles Traduits';

CREATE TABLE REPREV(
  Id NUMBER NOT NULL,
  IdVehicle NUMBER NOT NULL,
  IdCentre NUMBER NOT NULL,
  IdClient NUMBER,
  Incidencia VARCHAR2(3) NOT NULL, 
  IdAccioSol NUMBER,
  DataEntrada DATE DEFAULT SYSDATE NOT NULL,
  DataSortida DATE,
  DataInici DATE,
  DataFi DATE,
  IdMecanicEnt NUMBER NOT NULL,
  IdMecanic NUMBER,
  IdMecanicSor NUMBER,
  Preu NUMBER,
  ComentariClient VARCHAR2(2000),
  ComentariMecanic VARCHAR2(2000),
  CONSTRAINT PK_REPREV PRIMARY KEY(Id),
  CONSTRAINT FK_REPREV_VEHICLES FOREIGN KEY (IdVehicle) REFERENCES VEHICLES(Id),
  CONSTRAINT FK_REPREV_ACCIO FOREIGN KEY (IdAccioSol) REFERENCES REPREV_ACCIONS(Id),
  CONSTRAINT FK_REPREV_CENTRES FOREIGN KEY (IdCentre) REFERENCES CENTRES(Id),
  CONSTRAINT FK_REPREV_CLIENTS FOREIGN KEY (IdClient) REFERENCES CLIENTS(Id),
  CONSTRAINT FK_REPREV_MECENT FOREIGN KEY (IdMecanicEnt) REFERENCES TREBALLADORS(Id),
  CONSTRAINT FK_REPREV_MECANIC FOREIGN KEY (IdMecanic) REFERENCES TREBALLADORS(Id),
  CONSTRAINT FK_REPREV_MECSOR FOREIGN KEY (IdMecanicSor) REFERENCES TREBALLADORS(Id),
  CONSTRAINT CK_VEHICLES_INCIDENCIA CHECK (Incidencia IN ('REV', 'REP'))
);

COMMENT ON TABLE REPREV IS 'Taula de gesti i histric de dades generals de reparacions i revisions de vehicles';
COMMENT ON COLUMN REPREV.Incidencia IS 'Els valors possibles sn REV - Revisi, REP - Reparaci';
COMMENT ON COLUMN REPREV.IdAccioSol IS 'Acci detallada per la que el client porta el vehicle';

CREATE TABLE REPREV_LINIES(
  Id NUMBER NOT NULL,
  IdRepRev NUMBER NOT NULL,
  IdAccio NUMBER NOT NULL,
  Preu NUMBER NOT NULL,
  Quantitat NUMBER NOT NULL,
  Comentari VARCHAR2 (255),
  CONSTRAINT PK_REPREV_LINIES PRIMARY KEY(Id),
  CONSTRAINT FK_REPREV_LINIES_REPREV FOREIGN KEY (IdRepRev) REFERENCES REPREV(Id),
  CONSTRAINT FK_REPREV_LINIES_RREVACC FOREIGN KEY (IdAccio) REFERENCES REPREV_ACCIONS(Id) 
);

COMMENT ON TABLE REPREV_LINIES IS 'Taula de les linees de detall de cada revisi o reparaci';



CREATE TABLE VENTES(
  Id NUMBER NOT NULL,
  IdVehicle NUMBER NOT NULL,
  IdClient NUMBER NOT NULL,
  IdCentre NUMBER NOT NULL,
  IdTreballador NUMBER NOT NULL,
  DataEncarrec DATE,
  Pagat NUMBER,
  DataVenta DATE,
  Preu NUMBER NOT NULL,
  Descompte NUMBER DEFAULT 0 NOT NULL,
  PreuFinal NUMBER NOT NULL,
  DataPrevEntrega DATE,
  DataEntrega DATE,
  Us VARCHAR2(1) NOT NULL,
  DataBaixa DATE,
  Comentari VARCHAR2(255),
  CONSTRAINT PK_VENTES PRIMARY KEY(Id),
  CONSTRAINT FK_VENTES_VEHICLES FOREIGN KEY (IdVehicle) REFERENCES VEHICLES(Id),
  CONSTRAINT FK_VENTES_CLIENTS FOREIGN KEY (IdClient) REFERENCES CLIENTS(Id),
  CONSTRAINT FK_VENTES_CENTRES FOREIGN KEY (IdCentre) REFERENCES CENTRES(Id),
  CONSTRAINT FK_VENTES_TREBALLADORS FOREIGN KEY (IdTreballador) REFERENCES TREBALLADORS(Id), 
  CONSTRAINT CK_VENTES_US CHECK (Us IN ('N', 'S'))
);

COMMENT ON TABLE VENTES IS 'Taula de ventes i histric de ventes de vehicles';
COMMENT ON COLUMN VENTES.Us IS 'Els valors possibles sn N - Nou, S - Segona ma';

CREATE TABLE COMPRES(
  Id NUMBER NOT NULL,
  IdCentre NUMBER NOT NULL,
  IdTreballador NUMBER NOT NULL,
  IdDistribuidor NUMBER,
  IdClient NUMBER,
  Us VARCHAR2(1) NOT NULL,
  DataCompra DATE DEFAULT SYSDATE NOT NULL,
  DataTancament DATE,
  IdAutoritzador NUMBER,
  Preu NUMBER NOT NULL,
  DataRecepcio DATE,
  IdReceptor NUMBER,
  DataBaixa DATE,
  Comentari VARCHAR2(255),
  CONSTRAINT PK_COMPRES PRIMARY KEY(Id),
  CONSTRAINT FK_COMPRES_CENTRES FOREIGN KEY (IdCentre) REFERENCES CENTRES(Id),
  CONSTRAINT FK_COMPRES_TREBALLADORS FOREIGN KEY (IdTreballador) REFERENCES TREBALLADORS(Id),   
  CONSTRAINT FK_COMPRES_DISTRIBUIDORS FOREIGN KEY (IdDistribuidor) REFERENCES DISTRIBUIDORS(Id),
  CONSTRAINT FK_COMPRES_CLIENTS FOREIGN KEY (IdClient) REFERENCES CLIENTS(Id),
  CONSTRAINT FK_COMPRES_AUTORITZADOR FOREIGN KEY (IdAutoritzador) REFERENCES TREBALLADORS(Id),  
  CONSTRAINT FK_COMPRES_RECEPTOR FOREIGN KEY (Idreceptor) REFERENCES TREBALLADORS(Id),
  CONSTRAINT CK_COMPRES_US CHECK (Us IN ('N', 'S'))
);

COMMENT ON TABLE COMPRES IS 'Taula de compres de vehicles';
COMMENT ON COLUMN COMPRES.Us IS 'Els valors possibles sn N - Nou, S - Segona ma';

CREATE TABLE COMPRES_LINIES(
  Id NUMBER NOT NULL,
  IdCompra NUMBER NOT NULL,
  IdVehicle NUMBER NOT NULL,
  Preu NUMBER NOT NULL,
  Comentari VARCHAR2(255),
  CONSTRAINT PK_COMPRES_LINIES PRIMARY KEY(Id),
  CONSTRAINT FK_COMPRES_LINIES_COMPRES FOREIGN KEY (IdCompra) REFERENCES COMPRES(Id),
  CONSTRAINT FK_COMPRES_LINIES_VEHICLES FOREIGN KEY (IdVehicle) REFERENCES VEHICLES(Id)
);

COMMENT ON TABLE COMPRES_LINIES IS 'Taula de detall de les compres de vehicles';

CREATE TABLE STOCK(
  Id NUMBER NOT NULL,
  IdVehicle NUMBER NOT NULL,  
  IdCentre NUMBER NOT NULL,
  DataEntrada DATE DEFAULT SYSDATE NOT NULL,
  DataEncarrec DATE, 
  DataSortida DATE,
  PreuVenta NUMBER NOT NULL,
  Us VARCHAR2(1) NOT NULL,
  CONSTRAINT PK_STOCK PRIMARY KEY(Id),
  CONSTRAINT FK_STOCK_VEHICLES FOREIGN KEY (IdVehicle) REFERENCES VEHICLES(Id), 
  CONSTRAINT FK_STOCK_CENTRES FOREIGN KEY (IdCentre) REFERENCES CENTRES(Id),
  CONSTRAINT CK_STOCK_US CHECK (Us IN ('N', 'S'))
);

COMMENT ON TABLE STOCK IS 'Taula de control de vehicles estocats';
COMMENT ON COLUMN STOCK.Us IS 'Els valors possibles sn N - Nou, S - Segona ma';

CREATE TABLE HISTORIC(
  Id NUMBER NOT NULL,
  Taula VARCHAR2(30) NOT NULL,
  IdRegistre NUMBER NOT NULL,
  DataAccio DATE DEFAULT SYSDATE NOT NULL, 
  Accio VARCHAR2(1) NOT NULL,
  IdTreballador NUMBER NOT NULL,
  CONSTRAINT PK_HISTORIC PRIMARY KEY(Id),
  CONSTRAINT FK_HISTORIC_TREBAL FOREIGN KEY (IdTreballador) REFERENCES TREBALLADORS(Id),
  CONSTRAINT CK_HISTORIC_ACCIO CHECK (Accio IN ('I', 'U', 'D'))
);

COMMENT ON TABLE HISTORIC IS 'Taula de logs';
COMMENT ON COLUMN HISTORIC.Accio IS 'Els valors possibles sn I - Insert, U - Update o D - Delete';

CREATE TABLE HISTORIC_DETALL(
  Id NUMBER NOT NULL,
  IdHistoric NUMBER NOT NULL,
  Columna VARCHAR2(30) NOT NULL, 
  Valor VARCHAR2(255),
  CONSTRAINT PK_HISTORIC_DETALL PRIMARY KEY(Id),
  CONSTRAINT FK_HISTORIC_DETALL_HISTORIC FOREIGN KEY (IdHistoric) REFERENCES HISTORIC(Id)
);

COMMENT ON TABLE HISTORIC_DETALL IS 'Taula de detall de logs per esbrinar valors antics en cas de esborrat o modificaci';

CREATE TABLE LOGERROR(
  Id NUMBER NOT NULL,
  ErrorCode VARCHAR2(20) NOT NULL,
  Taula VARCHAR2(30),
  IdRegistre NUMBER,
  Descripcio VARCHAR2(255),
  DataError DATE DEFAULT SYSDATE NOT NULL, 
  Accio VARCHAR2(1) NOT NULL,
  IdTreballador NUMBER NOT NULL,
  CONSTRAINT PK_LOGERROR PRIMARY KEY(Id),
  CONSTRAINT FK_LOGERROR_TREBAL FOREIGN KEY (IdTreballador) REFERENCES TREBALLADORS(Id),
  CONSTRAINT CK_LOGERROR_ACCIO CHECK (Accio IN ('I', 'U', 'D', 'F', 'P', 'C', 'A'))
);

COMMENT ON TABLE LOGERROR IS 'Taula de logs';
COMMENT ON COLUMN LOGERROR.Accio IS 'Els valors possibles sn I - Insert, U - Update, D - Delete, F - Funci, P - Procs, C - Consulta, A - Altres';