create or replace
PACKAGE BODY PAK_CENTRES AS

  TE_FILLS exception;
  pragma exception_init( TE_FILLS, -2292 );

FUNCTION CENTRES_I (pTipus VARCHAR2, pNom VARCHAR2, pDocTipus VARCHAR2, pDocNumero VARCHAR2, 
                    pAdrea VARCHAR2, pCP VARCHAR2, pCiutat VARCHAR2, pProvincia VARCHAR2, 
                    pPais VARCHAR2, pDataBaixa DATE, pIdIdioma NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vAux NUMBER;
  BEGIN 

    IF FUNC_ACCES_PERMES ('CENTRES_I', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
    
    IF pNom IS NULL THEN RETURN RETORNS(2, 0, 'CENTRES.Nom');/*Valor Null no possible*/
    ELSIF pTipus IS NULL THEN RETURN RETORNS(2, 0, 'CENTRES.Tipus');/*Valor Null no possible*/
    ELSIF pTipus  NOT IN ('T', 'C') THEN RETURN RETORNS(11, 0, 'CENTRES.Tipus');/*Error Tipus de centres*/
    ELSIF pDocTipus IS NULL THEN RETURN RETORNS(2, 0, 'CENTRES.DocTipus');/*Valor Null no possible*/ 
    ELSIF pDocTipus NOT IN ('NIF', 'CIF', 'NIE', 'PASSPORT') THEN RETURN RETORNS(7, 0, 'CENTRES.DocTipus');/*Error Tipus Document*/     
    ELSIF pDocNumero IS NULL THEN RETURN RETORNS(2, 0, 'CENTRES.DocNumero');/*Valor Null no possible*/ 
    END IF;
    
    IF pIdIdioma IS NULL  THEN
      RETURN RETORNS(2, 0, 'TREBALLADORS.IdIdioma');/*Valor Null no possible*/ 
    ELSE
      BEGIN
        SELECT Id
          INTO vAux
          FROM IDIOMES
         WHERE Id = pIdIdioma;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdIdioma, 'TREBALLADORS.IdIdioma');/*Valor d'Id No existeix*/
      END;
    END IF;
    
    INSERT INTO CENTRES (Id, Tipus, Nom, DocTipus, DocNumero, Adrea, CP, Ciutat, Provincia, Pais, DataBaixa, IdIdioma)
                        VALUES (SEQ_CENTRES.NEXTVAL, pTipus, pNom, pDocTipus, 
                                pDocNumero, pAdrea, pCP, pCiutat, pProvincia, pPais, pDataBaixa, pIdIdioma)
    RETURNING Id INTO vRes;

    RETURN RETORNS(0, vRes,null);
 
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CENTRES', vRes, SQLCODE, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

FUNCTION CENTRES_CONTACTE_I (pIdCen NUMBER, pTipus VARCHAR2, pDada VARCHAR2, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vAux NUMBER;
  BEGIN 

    IF FUNC_ACCES_PERMES ('CENTRES_CONTACTE_I', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
    
    IF pIdCen IS NULL THEN RETURN RETORNS(2, 0, 'CENTRES_CONTACTE.IdCentre');/*Valor Null no possible*/
    ELSIF pTipus IS NULL THEN RETURN RETORNS(2, 0, 'CENTRES_CONTACTE.Tipus');/*Valor Null no possible*/
    ELSIF pTipus NOT IN ('TP', 'TT', 'TM', 'FA', 'EP', 'ET') THEN RETURN RETORNS(9, 0, 'CENTRES_CONTACTE.Tipus');/*Error Tipus Contacte*/     
    ELSIF pDada IS NULL THEN RETURN RETORNS(2, 0, 'CENTRES_CONTACTE.Dada');/*Valor Null no possible*/
    END IF;

    BEGIN
      SELECT Id
        INTO vAux
        FROM CENTRES
       WHERE Id = pIdCen;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pIdCen, 'CENTRES_IDIOMA.IdCentre');/*Valor d'Id No existeix*/
    END;

    INSERT INTO CENTRES_CONTACTE VALUES (SEQ_CENTRES_CONTACTE.NEXTVAL, pIdCen, pTipus, pDada)
    RETURNING Id INTO VRes;

    RETURN RETORNS(0, vRes, null);

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CENTRES_CONTACTE', vRes, SQLCODE, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION CENTRES_U (pId NUMBER, pTipus VARCHAR2, pNom VARCHAR2, pDocTipus VARCHAR2, pDocNumero VARCHAR2, 
                      pAdrea VARCHAR2, pCP VARCHAR2, pCiutat VARCHAR2, pProvincia VARCHAR2, 
                      pPais VARCHAR2, pDataBaixa DATE, pIdIdioma NUMBER, pIdUser NUMBER) RETURN RETORNS  IS
    vAux VARCHAR2 (100);
    vModif BOOLEAN := FALSE;
    vFila CENTRES%ROWTYPE;
    vSQL VARCHAR2(1000):='';
  BEGIN 

    IF FUNC_ACCES_PERMES ('CENTRES_U', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 

    IF pTipus IS NOT NULL AND pTipus  NOT IN ('T', 'C') THEN
      RETURN RETORNS(11, 0, 'CENTRES.Tipus');/*Error Tipus de centres*/
    ELSIF pDocTipus IS NOT NULL AND pDocTipus NOT IN ('NIF', 'CIF', 'NIE', 'PASSPORT') THEN 
      RETURN RETORNS(7, 0, 'CENTRES.DocTipus');
    END IF;
    
  IF pId IS NULL THEN 
     RETURN RETORNS(2, 0, 'CENTRES.Id');/*Valor Null no possible*/
   ELSE
     BEGIN
       SELECT *
         INTO vFila
         FROM CENTRES
        WHERE Id = pId;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
         RETURN RETORNS(3,pId, 'CENTRES.Id');/*Valor d'Id No existeix*/
     END;
   END IF;

    IF pTipus IS NOT NULL AND pTipus <> vFila.Tipus THEN
      vSQL := vSQL||' Tipus = ''' || pTipus ||''',';
      vModif := TRUE;
    END IF;

    IF pNom IS NOT NULL AND pNom <> vFila.Nom THEN
      vSQL := vSQL||' Nom = ''' || pNom ||''',';
      vModif := TRUE;
    END IF;

    IF pDocTipus IS NOT NULL AND pDocTipus <> vFila.DocTipus THEN
      vSQL := vSQL||' DocTipus = ''' || pDocTipus ||''',';
      vModif := TRUE;
    END IF;
    
    IF pDocNumero IS NOT NULL AND pDocNumero <> vFila.DocNumero THEN
      vSQL := vSQL||' DocNumero = ''' || pDocNumero ||''',';
      vModif := TRUE;
    END IF;   
    
    IF pAdrea IS NOT NULL AND pAdrea <> vFila.Adrea THEN
      vSQL := vSQL||' Adrea = ''' || pAdrea ||''',';
      vModif := TRUE;
    END IF;  

    IF pCP IS NOT NULL AND pCP <> vFila.CP THEN
      vSQL := vSQL||' CP = ''' || pCP ||''',';
      vModif := TRUE;
    END IF;   
    
    IF pCiutat IS NOT NULL AND pCiutat <> vFila.Ciutat THEN
      vSQL := vSQL||' Ciutat = ''' || pCiutat ||''',';
      vModif := TRUE;
    END IF; 
    
    IF pProvincia IS NOT NULL AND pProvincia <> vFila.Provincia THEN
      vSQL := vSQL||' Provincia = ''' || pProvincia ||''',';
      vModif := TRUE;
    END IF;   
    
    IF pPais IS NOT NULL AND pPais <> vFila.Pais THEN
      vSQL := vSQL||' Pais = ''' || pPais ||''',';
      vModif := TRUE;
    END IF; 
    
    IF pDataBaixa IS NOT NULL AND pDataBaixa <> vFila.DataBaixa THEN
      vSQL := vSQL||' DataBaixa = ''' || pDataBaixa ||''',';
      vModif := TRUE;
    END IF; 
        
    IF pIdIdioma IS NOT NULL AND pIdIdioma <> vFila.IdIdioma THEN
      BEGIN
        SELECT Id
          INTO vAux
          FROM IDIOMES
         WHERE Id = pIdIdioma;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdIdioma, 'CENTRES.IdIdioma');/*Valor d'Id No existeix*/
      END;
      vSQL := vSQL||' IdIdioma = ' || pIdIdioma||',';
      vModif := TRUE;
    END IF;
  
    IF vModif THEN
      vSQL := 'UPDATE CENTRES SET ' || SUBSTR(vSQL, 1,LENGTH (vSQL)-1)|| ' WHERE Id = '|| pId;
      EXECUTE IMMEDIATE (vSQL);

      IF sql%ROWCOUNT >=1 THEN
        RETURN RETORNS(0, pId, null);
      ELSE
        RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
      END IF;
    ELSE
      RETURN RETORNS(4, pId, null);/*No s'ha realitzat cap acci*/
    END IF;
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CENTRES', pId, vSQL, 'U', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION CENTRES_CONTACTE_U (pId NUMBER, pIdCen NUMBER, pTipus VARCHAR2, pDada VARCHAR2, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
    vModif BOOLEAN := FALSE;
    vFila CENTRES_CONTACTE%ROWTYPE;
    vSQL VARCHAR2(1000):='';
  BEGIN 

    IF FUNC_ACCES_PERMES ('CENTRES_CONTACTE_U', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 

    IF pTipus IS NOT NULL AND pTipus NOT IN ('TP', 'TT', 'TM', 'FA', 'EP', 'ET') THEN 
      RETURN RETORNS(9, 0, 'CENTRES_CONTACTE.Tipus');/*Error Tipus Contacte*/     
    END IF;

    
   IF pId IS NULL THEN 
     RETURN RETORNS(2, 0, 'CENTRES_CONTACTE.Id');/*Valor Null no possible*/
   ELSE
     BEGIN
       SELECT *
         INTO vFila
         FROM CENTRES_CONTACTE
        WHERE Id = pId;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
         RETURN RETORNS(3,pId, 'CENTRES_CONTACTE.Id');/*Valor d'Id No existeix*/
     END;
   END IF;
   
    IF pIdCen IS NOT NULL AND pIdCen <> vFila.IdCentre THEN
      BEGIN
        SELECT Id
          INTO vAux
          FROM CENTRES
         WHERE Id = pIdCen;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdCen, 'CENTRES_CONTACTE.IdCentre');/*Valor d'Id No existeix*/
      END;
      vSQL := vSQL||' IdCentre = ' || pIdCen||',';
      vModif := TRUE;
    END IF;
  
    IF pTipus IS NOT NULL AND pTipus <> vFila.Tipus THEN
      vSQL := vSQL||' Tipus = ''' || pTipus ||''',';
      vModif := TRUE;
    END IF;


    IF pDada <> vFila.Dada THEN
      vSQL := vSQL||' Dada = ''' || pDada||''',';
      vModif := TRUE;
    END IF;

    IF vModif THEN
      vSQL := 'UPDATE CENTRES_CONTACTE SET ' || SUBSTR(vSQL, 1,LENGTH (vSQL)-1)|| ' WHERE Id = '|| pId;
      EXECUTE IMMEDIATE (vSQL);

      IF sql%ROWCOUNT >=1 THEN
        RETURN RETORNS(0, pId, null);
      ELSE
        RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
      END IF;
    ELSE
      RETURN RETORNS(4, pId, null);/*No s'ha realitzat cap acci*/
    END IF;
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CENTRES_CONTACTE', pId, vSQL, 'U', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

FUNCTION CENTRES_D (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
  BEGIN 
 
    IF FUNC_ACCES_PERMES ('CENTRES_D', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
     
    BEGIN
      SELECT Id
        INTO vAux
        FROM CENTRES
       WHERE Id = pId;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pId, 'CENTRES.Id');/*Valor d'Id No existeix*/
    END;
    
    DELETE CENTRES WHERE Id = pId;
    
    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha esborrat registre*/
    END IF;
  
  EXCEPTION
    WHEN TE_FILLS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CENTRES', pId, 'FK Error: '||SQLERRM, 'D', 0);
      RETURN RETORNS(6,pId, 'CENTRES.Id');/*Error general revisar LOG*/
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CENTRES', pId, SQLERRM, 'D', 0);
      RETURN RETORNS(1,pId, SQLCODE);/*Error general revisar LOG*/
  END;
  
  FUNCTION CENTRES_CONTACTE_D (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
  BEGIN 
 
    IF FUNC_ACCES_PERMES ('CENTRES_CONTACTE_D', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
     
    IF pId IS NULL THEN
      RETURN RETORNS(2, 0, 'CENTRES_CONTACTE.Id');/*Valor Null no possible*/
    END IF;
  
    BEGIN
      SELECT Id
        INTO vAux
        FROM CENTRES_CONTACTE
       WHERE Id = pId;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pId, 'CENTRES_CONTACTE.pId');/*Valor d'Id No existeix*/
    END;
  
    DELETE CENTRES_CONTACTE WHERE Id = pId;
    
    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha esborrat registre*/
    END IF;
  
  EXCEPTION
   WHEN TE_FILLS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CENTRES_CONTACTE', pId, 'FK Error: '||SQLERRM, 'D', 0);
      RETURN RETORNS(6,pId, 'CENTRES_CONTACTE.Id');/*Error general revisar LOG*/  
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CENTRES_CONTACTE', pId, SQLERRM, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

END PAK_CENTRES;
/