create or replace
PACKAGE BODY PAK_CLIENTS AS

  TE_FILLS exception;
  pragma exception_init( TE_FILLS, -2292 );

  FUNCTION CLIENTS_I (pNom VARCHAR2, pCognom1 VARCHAR2, pCognom2 VARCHAR2, pDataNaixement DATE, pSexe VARCHAR2, 
                      pDocTipus VARCHAR2, pDocNumero VARCHAR2, pAdrea VARCHAR2, pCP VARCHAR2, pCiutat VARCHAR2, 
                      pProvincia VARCHAR2, pPais VARCHAR2, pDataBaixa DATE, pIdIdioma NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vAux NUMBER;
  BEGIN 

    IF FUNC_ACCES_PERMES ('CLIENTS_I', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
    
    IF pNom IS NULL THEN RETURN RETORNS(2, 0, 'CLIENTS.Nom');/*Valor Null no possible*/
    ELSIF pCognom1 IS NULL THEN RETURN RETORNS(2, 0, 'CLIENTS.Cognom1');/*Valor Null no possible*/
    ELSIF pDocTipus IS NULL THEN RETURN RETORNS(2, 0, 'CLIENTS.DocTipus');/*Valor Null no possible*/ 
    ELSIF pDocTipus NOT IN ('NIF', 'CIF', 'NIE', 'PASSPORT') THEN RETURN RETORNS(7, 0, 'CLIENTS.DocTipus');/*Error Tipus Document*/     
    ELSIF pSexe IS NULL THEN RETURN RETORNS(2, 0, 'CLIENTS.Sexe');/*Valor Null no possible*/
    ELSIF pSexe NOT IN ('H', 'D') THEN RETURN RETORNS(56, 0, 'CLIENTS.Sexe');
    ELSIF pDocNumero IS NULL THEN RETURN RETORNS(2, 0, 'CLIENTS.DocNumero');/*Valor Null no possible*/ 
    END IF;
    
    IF pIdIdioma IS NULL  THEN
      RETURN RETORNS(2, 0, 'TREBALLADORS.IdIdioma');/*Valor Null no possible*/ 
    ELSE
      BEGIN
        SELECT Id
          INTO vAux
          FROM IDIOMES
         WHERE Id = pIdIdioma;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdIdioma, 'TREBALLADORS.IdIdioma');/*Valor d'Id No existeix*/
      END;
    END IF;  
    INSERT INTO CLIENTS (Id, Nom, Cognom1, Cognom2, DataNaixement, Sexe, DocTipus, 
                         DocNumero, Adrea, CP, Ciutat, Provincia, Pais, DataBaixa, IdIdioma)
                        VALUES (SEQ_CLIENTS.NEXTVAL, pNom, pCognom1, pCognom2, pDataNaixement, pSexe, 
                                pDocTipus, pDocNumero, pAdrea, pCP, pCiutat, pProvincia, pPais, pDataBaixa, pIdIdioma)
    RETURNING Id INTO vRes;

    RETURN RETORNS(0, vRes,null);
 
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CLIENTS', vRes, SQLCODE, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

FUNCTION CLIENTS_CONTACTE_I (pIdCli NUMBER, pTipus VARCHAR2, pDada VARCHAR2, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vAux NUMBER;
  BEGIN 

    IF FUNC_ACCES_PERMES ('CLIENTS_CONTACTE_I', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
    
    IF pIdCli IS NULL THEN RETURN RETORNS(2, 0, 'CLIENTS_CONTACTE.IdClient');/*Valor Null no possible*/
    ELSIF pTipus IS NULL THEN RETURN RETORNS(2, 0, 'CLIENTS_CONTACTE.Tipus');/*Valor Null no possible*/
    ELSIF pTipus NOT IN ('TP', 'TT', 'TM', 'FA', 'EP', 'ET') THEN RETURN RETORNS(9, 0, 'CLIENTS_CONTACTE.Tipus');/*Error Tipus Contacte*/     
    ELSIF pDada IS NULL THEN RETURN RETORNS(2, 0, 'CLIENTS_CONTACTE.Dada');/*Valor Null no possible*/
    END IF;

    BEGIN
      SELECT Id
        INTO vAux
        FROM CLIENTS
       WHERE Id = pIdCli;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pIdCli, 'CLIENTS_CONTACTE.IdClient');/*Valor d'Id No existeix*/
    END;

    INSERT INTO CLIENTS_CONTACTE VALUES (SEQ_CLIENTS_CONTACTE.NEXTVAL, pIdCli, pTipus, pDada)
    RETURNING Id INTO VRes;

    RETURN RETORNS(0, vRes, null);

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CLIENTS_CONTACTE', vRes, SQLCODE, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION CLIENTS_U (pId NUMBER, pNom VARCHAR2, pCognom1 VARCHAR2, pCognom2 VARCHAR2, pDataNaixement DATE, 
                      pSexe VARCHAR2,pDocTipus VARCHAR2, pDocNumero VARCHAR2, pAdrea VARCHAR2, pCP VARCHAR2, pCiutat VARCHAR2, pProvincia VARCHAR2, 
                    pPais VARCHAR2, pDataBaixa DATE, pIdIdioma NUMBER, pIdUser NUMBER) RETURN RETORNS  IS
    vAux VARCHAR2 (100);
    vModif BOOLEAN := FALSE;
    vFila CLIENTS%ROWTYPE;
    vSQL VARCHAR2(1000):='';
  BEGIN 

    IF FUNC_ACCES_PERMES ('CLIENTS_U', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
    
    IF pDocTipus IS NOT NULL AND pDocTipus NOT IN ('NIF', 'CIF', 'NIE', 'PASSPORT') THEN 
      RETURN RETORNS(7, 0, 'CLIENTS.DocTipus');
    END IF;
    
    IF pSexe IS NOT NULL AND pSexe NOT IN ('H', 'D') THEN 
      RETURN RETORNS(56, 0, 'CLIENTS.Sexe');
    END IF;
    
  IF pId IS NULL THEN 
     RETURN RETORNS(2, 0, 'CLIENTS.Id');/*Valor Null no possible*/
   ELSE
     BEGIN
       SELECT *
         INTO vFila
         FROM CLIENTS
        WHERE Id = pId;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
         RETURN RETORNS(3,pId, 'CLIENTS.Id');/*Valor d'Id No existeix*/
     END;
   END IF;

    IF pNom IS NOT NULL AND pNom <> vFila.Nom THEN
      vSQL := vSQL||' Nom = ''' || pNom ||''',';
      vModif := TRUE;
    END IF;

    IF pCognom1 IS NOT NULL AND pCognom1 <> vFila.Cognom1 THEN
      vSQL := vSQL||' Cognom1 = ''' || pCognom1 ||''',';
      vModif := TRUE;
    END IF;
    
    IF pCognom2 IS NOT NULL AND pCognom2 <> vFila.Cognom2 THEN
      vSQL := vSQL||' Cognom2 = ''' || pCognom2 ||''',';
      vModif := TRUE;
    END IF;

    IF pDataNaixement IS NOT NULL AND pDataNaixement <> vFila.DataNaixement THEN
      vSQL := vSQL||' DataNaixement = ''' || pDataNaixement ||''',';
      vModif := TRUE;
    END IF;
    
    IF pSexe IS NOT NULL AND pSexe <> vFila.Sexe THEN
      vSQL := vSQL||' Sexe = ''' || pSexe ||''',';
      vModif := TRUE;
    END IF;

    IF pDocTipus IS NOT NULL AND pDocTipus <> vFila.DocTipus THEN
      vSQL := vSQL||' DocTipus = ''' || pDocTipus ||''',';
      vModif := TRUE;
    END IF;
    
    IF pDocNumero IS NOT NULL AND pDocNumero <> vFila.DocNumero THEN
      vSQL := vSQL||' DocNumero = ''' || pDocNumero ||''',';
      vModif := TRUE;
    END IF;   
    
    IF pAdrea IS NOT NULL AND pAdrea <> vFila.Adrea THEN
      vSQL := vSQL||' Adrea = ''' || pAdrea ||''',';
      vModif := TRUE;
    END IF;  

    IF pCP IS NOT NULL AND pCP <> vFila.CP THEN
      vSQL := vSQL||' CP = ''' || pCP ||''',';
      vModif := TRUE;
    END IF;   
    
    IF pCiutat IS NOT NULL AND pCiutat <> vFila.Ciutat THEN
      vSQL := vSQL||' Ciutat = ''' || pCiutat ||''',';
      vModif := TRUE;
    END IF; 
    
    IF pProvincia IS NOT NULL AND pProvincia <> vFila.Provincia THEN
      vSQL := vSQL||' Provincia = ''' || pProvincia ||''',';
      vModif := TRUE;
    END IF;   
    
    IF pPais IS NOT NULL AND pPais <> vFila.Pais THEN
      vSQL := vSQL||' Pais = ''' || pPais ||''',';
      vModif := TRUE;
    END IF; 
    
    IF pDataBaixa IS NOT NULL AND pDataBaixa <> vFila.DataBaixa THEN
      vSQL := vSQL||' DataBaixa = ''' || pDataBaixa ||''',';
      vModif := TRUE;
    END IF; 
    
    IF pIdIdioma IS NOT NULL AND pIdIdioma <> vFila.IdIdioma THEN
      BEGIN
        SELECT Id
          INTO vAux
          FROM IDIOMES
         WHERE Id = pIdIdioma;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdIdioma, 'CLIENTS.IdIdioma');/*Valor d'Id No existeix*/
      END;
      vSQL := vSQL||' IdIdioma = ' || pIdIdioma||',';
      vModif := TRUE;
    END IF;
  
    IF vModif THEN
      vSQL := 'UPDATE CLIENTS SET ' || SUBSTR(vSQL, 1,LENGTH (vSQL)-1)|| ' WHERE Id = '|| pId;
      EXECUTE IMMEDIATE (vSQL);

      IF sql%ROWCOUNT >=1 THEN
        RETURN RETORNS(0, pId, null);
      ELSE
        RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
      END IF;
    ELSE
      RETURN RETORNS(4, pId, null);/*No s'ha realitzat cap acci*/
    END IF;
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CLIENTS', pId, vSQL, 'U', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION CLIENTS_CONTACTE_U (pId NUMBER, pIdCli NUMBER, pTipus VARCHAR2, pDada VARCHAR2, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
    vModif BOOLEAN := FALSE;
    vFila CLIENTS_CONTACTE%ROWTYPE;
    vSQL VARCHAR2(1000):='';
  BEGIN 

    IF FUNC_ACCES_PERMES ('CLIENTS_CONTACTE_U', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 

    IF pTipus IS NOT NULL AND pTipus NOT IN ('TP', 'TT', 'TM', 'FA', 'EP', 'ET') THEN 
      RETURN RETORNS(9, 0, 'CLIENTS_CONTACTE.Tipus');/*Error Tipus Contacte*/     
    END IF;
    
   IF pId IS NULL THEN 
     RETURN RETORNS(2, 0, 'CLIENTS_CONTACTE.Id');/*Valor Null no possible*/
   ELSE
     BEGIN
       SELECT *
         INTO vFila
         FROM CLIENTS_CONTACTE
        WHERE Id = pId;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
         RETURN RETORNS(3,pId, 'CLIENTS_CONTACTE.Id');/*Valor d'Id No existeix*/
     END;
   END IF;
   
    IF pIdCli IS NOT NULL AND pIdCli <> vFila.IdClient THEN
      BEGIN
        SELECT Id
          INTO vAux
          FROM CLIENTS
         WHERE Id = pIdCli;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdCli, 'CLIENTS_CONTACTE.IdClient');/*Valor d'Id No existeix*/
      END;
      vSQL := vSQL||' IdClient = ' || pIdCli||',';
      vModif := TRUE;
    END IF;
  
    IF pTipus IS NOT NULL AND pTipus <> vFila.Tipus THEN
      vSQL := vSQL||' Tipus = ''' || pTipus ||''',';
      vModif := TRUE;
    END IF;


    IF pDada <> vFila.Dada THEN
      vSQL := vSQL||' Dada = ''' || pDada||''',';
      vModif := TRUE;
    END IF;

    IF vModif THEN
      vSQL := 'UPDATE CLIENTS_CONTACTE SET ' || SUBSTR(vSQL, 1,LENGTH (vSQL)-1)|| ' WHERE Id = '|| pId;
      EXECUTE IMMEDIATE (vSQL);

      IF sql%ROWCOUNT >=1 THEN
        RETURN RETORNS(0, pId, null);
      ELSE
        RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
      END IF;
    ELSE
      RETURN RETORNS(4, pId, null);/*No s'ha realitzat cap acci*/
    END IF;
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CLIENTS_CONTACTE', pId, vSQL, 'U', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

FUNCTION CLIENTS_D (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
  BEGIN 
 
    IF FUNC_ACCES_PERMES ('CLIENTS_D', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
     
    BEGIN
      SELECT Id
        INTO vAux
        FROM CLIENTS
       WHERE Id = pId;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pId, 'CLIENTS.Id');/*Valor d'Id No existeix*/
    END;
    
    DELETE CLIENTS WHERE Id = pId;
    
    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha esborrat registre*/
    END IF;
  
  EXCEPTION
    WHEN TE_FILLS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CLIENTS', pId, 'FK Error: '||SQLERRM, 'D', 0);
      RETURN RETORNS(6,pId, 'CLIENTS.Id');/*Error general revisar LOG*/
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CLIENTS', pId, SQLERRM, 'D', 0);
      RETURN RETORNS(1,pId, SQLCODE);/*Error general revisar LOG*/
  END;
  
  FUNCTION CLIENTS_CONTACTE_D (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
  BEGIN 
 
    IF FUNC_ACCES_PERMES ('CLIENTS_CONTACTE_D', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
     
    IF pId IS NULL THEN
      RETURN RETORNS(2, 0, 'CLIENTS_CONTACTE.Id');/*Valor Null no possible*/
    END IF;
  
    BEGIN
      SELECT Id
        INTO vAux
        FROM CLIENTS_CONTACTE
       WHERE Id = pId;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pId, 'CLIENTS_CONTACTE.pId');/*Valor d'Id No existeix*/
    END;
  
    DELETE CLIENTS_CONTACTE WHERE Id = pId;
    
    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha esborrat registre*/
    END IF;
  
  EXCEPTION
   WHEN TE_FILLS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CLIENTS_CONTACTE', pId, 'FK Error: '||SQLERRM, 'D', 0);
      RETURN RETORNS(6,pId, 'CLIENTS_CONTACTE.Id');/*Error general revisar LOG*/  
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CLIENTS_CONTACTE', pId, SQLERRM, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION CLIENTS_BY_ID (pId NUMBER, pClient OUT CLIENTS_TYP, pIdUser NUMBER) RETURN RETORNS IS
    ret RETORNS;
  BEGIN
  
    IF FUNC_ACCES_PERMES ('CLIENTS_BY_ID', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
  
    BEGIN
      SELECT Id, Nom, Cognom1, Cognom2, 
             DocTipus, DocNumero, Adrea, CP, 
             Ciutat, Provincia, Pais, IdIdioma, DataBaixa
        INTO pClient.Id, pClient.Nom, pClient.Cognom1, pClient.Cognom2, 
             pClient.DocTipus, pClient.DocNumero, pClient.Adrea, pClient.CP, 
             pClient.Ciutat, pClient.Provincia, pClient.Pais, pClient.IdIdioma, pClient.DataBaixa 
        FROM CLIENTS 
       WHERE Id = pId;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pId, 'CLIENTS.pId');/*Valor d'Id No existeix*/
    END;
    ret:= CLIENTS_CONTACTE_BY_CLIENTS (pId, pClient.contactes, pIdUser);
    IF ret.IdMissatge = 0 THEN
     RETURN RETORNS(0,pId, 'CLIENTS.Id');
    ELSE 
      RETURN ret;
    END IF;
  
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CLIENTS', pId, SQLERRM, 'C', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;
 
  FUNCTION CLIENTS_CONTACTE_BY_ID (pId NUMBER, pDadesContacte OUT CONTACTE_TYP, pIdUser NUMBER) RETURN RETORNS IS

  BEGIN

    IF FUNC_ACCES_PERMES ('CLIENTS_CONTACTE_BY_ID', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 

    BEGIN
      SELECT Id, IdClient, Tipus, Dada
        INTO pDadesContacte.Id, pDadesContacte.IdPare, pDadesContacte.Tipus, pDadesContacte.Dada
        FROM CLIENTS_CONTACTE
       WHERE Id = pId;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pId, 'CLIENTS_CONTACTE.pId');/*Valor d'Id No existeix*/
    END;

     RETURN RETORNS(0,pId, 'CLIENTS_CONTACTE.Id');
  
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CLIENTS_CONTACTE', pId, SQLERRM, 'C', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

   FUNCTION CLIENTS_CONTACTE_BY_CLIENTS (pIdCli NUMBER, pDadesContacte OUT CONTACTE_TAB, pIdUser NUMBER) RETURN RETORNS IS
  
    CURSOR cContacte IS
      SELECT *
        FROM CLIENTS_CONTACTE
       WHERE IdClient = pIdCli;
  
    vContacte CLIENTS_CONTACTE%ROWTYPE;
    i NUMBER :=1;
  BEGIN

    IF FUNC_ACCES_PERMES ('CLIENTS_CONTACTE_BY_CLIENTS', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    OPEN cContacte;
      FETCH cContacte INTO vContacte;
      pDadesContacte  := CONTACTE_TAB();
      WHILE cContacte%found LOOP
        pDadesContacte.EXTEND;
        pDadesContacte(i).Id := vContacte.Id;
        pDadesContacte(i).IdPare := vContacte.IdClient;
        pDadesContacte(i).Tipus := vContacte.Tipus;
        pDadesContacte(i).Dada := vContacte.Dada;
        FETCH cContacte INTO vContacte;
        i := i + 1;
      END LOOP;
    CLOSE cContacte;
  
    RETURN RETORNS(0,pIdCli, 'CLIENTS_CONTACTES.IdClient');
  
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CLIENTS_CONTACTE', pIdCli, SQLERRM, 'C', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

END PAK_CLIENTS;
/