create or replace
PACKAGE BODY PAK_COMPRES AS

  TE_FILLS exception;
  pragma exception_init( TE_FILLS, -2292 );
  
  FUNCTION COMPRES_I (pIdCentre NUMBER, pIdTreballador NUMBER, pIdDistribuidor NUMBER, pIdClient NUMBER,
                    pUs VARCHAR2, pComentari VARCHAR2, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vAux VARCHAR2 (100);
  BEGIN 

    IF FUNC_ACCES_PERMES ('COMPRES_I', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 

    IF pUs IS NULL THEN 
      RETURN RETORNS(2, 0, 'COMPRES.Us');/*Valor Null no possible*/ 
    ELSIF pUs NOT IN ('N', 'S') THEN 
      RETURN RETORNS(28, 0, 'COMPRES.Us');/*Error Tipus Document*/     
    END IF;

    IF pIdCentre IS NULL THEN 
      RETURN RETORNS(2, 0, 'COMPRES.IdCentre'); /*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT Id
          INTO vAux
          FROM CENTRES
         WHERE Id = pIdCentre;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdCentre, 'COMPRES.IdCentre');/*Valor d'Id No existeix*/
      END;
    END IF;

   IF pIdTreballador IS NULL THEN 
      RETURN RETORNS(2, 0, 'COMPRES.pIdTreballador'); /*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT Id
          INTO vAux
          FROM TREBALLADORS
         WHERE Id = pIdTreballador;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdTreballador, 'COMPRES.IdTreballador');/*Valor d'Id No existeix*/
      END;
    END IF;

    IF pIdClient IS NULL AND pIdDistribuidor IS NULL THEN 
      RETURN RETORNS(26, 0, 'REPREV.IdClient'); /*Valor Null no possible*/
    ELSIF pIdClient IS NOT NULL AND pIdDistribuidor IS NOT NULL THEN
      RETURN RETORNS(27, 0, 'REPREV.IdClient'); /*Valor Null no possible*/
    ELSE
      IF pIdClient IS NOT NULL THEN
        BEGIN
          SELECT Id
            INTO vAux
            FROM CLIENTS
           WHERE Id = pIdClient;
        EXCEPTION
          WHEN NO_DATA_FOUND THEN
            RETURN RETORNS(3,pIdClient, 'COMPRES.IdClient');/*Valor d'Id No existeix*/
        END;
      ELSIF pIdDistribuidor IS NOT NULL THEN
        BEGIN
          SELECT Id
            INTO vAux
            FROM DISTRIBUIDORS
           WHERE Id = pIdDistribuidor;
        EXCEPTION
          WHEN NO_DATA_FOUND THEN
            RETURN RETORNS(3,pIdDistribuidor, 'COMPRES.IdDistribuidor');/*Valor d'Id No existeix*/
        END;
      END IF;
    END IF;

    INSERT INTO COMPRES (Id, IdCentre, IdTreballador, IdDistribuidor, IdClient, Us, Preu, Comentari)
                VALUES (SEQ_COMPRES.NEXTVAL, pIdCentre, pIdTreballador, pIdDistribuidor, pIdClient, pUs, 0, pComentari)
    RETURNING Id INTO vRes;

    RETURN RETORNS(0, vRes,null);
  
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'COMPRES', vRes, SQLERRM, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

 FUNCTION COMPRES_LINIES_I (pIdCom NUMBER, pIdVehicle NUMBER, pComentari VARCHAR2, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vFila COMPRES%ROWTYPE;
    vAux VARCHAR2 (100);
    vPreu NUMBER;
    
  BEGIN 
    
    IF FUNC_ACCES_PERMES ('COMPRES_LINIES_I', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 


    IF pIdCom IS NULL THEN 
      RETURN RETORNS(2, 0, 'COMPRES_LINIES.IdCompra');/*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT *
          INTO vFila
          FROM COMPRES
         WHERE Id = pIdCom;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdCom, 'COMPRES.IdCompra');/*Valor d'Id No existeix*/
      END;
    END IF;
    /* es comproba data de tancament, per que si es recepcionada ha de ser tancada*/
    IF vFila.DataTancament IS NOT NULL THEN
      RETURN RETORNS (33,0,'COMPRES.DataTancament');
    END IF;    

    IF vFila.DataBaixa IS NOT NULL THEN
      RETURN RETORNS (34,0,'COMPRES.DataBaixa');
    END IF;
    
    IF pIdVehicle IS NULL THEN 
      RETURN RETORNS(2, 0, 'COMPRES.IdVehicle');/*Valor Null no possible*/
    ELSE     
      BEGIN
        SELECT Id
          INTO vAux
          FROM VEHICLES
         WHERE Id = pIdVehicle;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdVehicle, 'COMPRES.IdVehicle');/*Valor d'Id No existeix*/
      END;
    END IF;   

      BEGIN 
       SELECT SUM (PreuF)
        INTO vPreu
        FROM ( SELECT B.Preu AS PreuF
                 FROM VEHICLES A, CONFIGFIN B
                WHERE A.Id = pIdVehicle AND
                      B.Id = A.IdConfigFin) 
           UNION(
               SELECT B.Preu AS PreuF
                FROM VEHICLES_EXTRES A, EXTRES B
               WHERE A.idVehicle = pIdVehicle AND
                     B.Id = A.IdExtra);       
              
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdVehicle, 'COMPRES.IdVehicle');/*Valor d'Id No existeix*/
      END;
      
      
      BEGIN
        vAux:= NULL;
        SELECT Id
          INTO vAux
          FROM COMPRES_LINIES
         WHERE IdVehicle = pIdVehicle AND
               IdCompra = pIdCom;
      IF vAux IS NOT NULL THEN
        RETURN RETORNS(35,pIdCom, 'COMPRES.IdVehicle');/*Vehicle ja en linia*/
      END IF;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
         null; /*tot OK*/
      END;
    
    INSERT INTO COMPRES_LINIES (Id, IdCompra, IdVehicle, Preu, Comentari)
                VALUES (SEQ_COMPRES_LINIES.NEXTVAL, pIdCom, pIdVehicle, vPreu, pComentari)
    RETURNING Id INTO vRes;

    RETURN RETORNS(0, vRes,null);

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'COMPRES_LINIES_I', vRes, SQLERRM, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;
    
  FUNCTION COMPRES_D (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
    vFila COMPRES%ROWTYPE;
  BEGIN 
 
    IF FUNC_ACCES_PERMES ('COMPRES_D', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 

    IF pId IS NULL THEN 
      RETURN RETORNS(2, 0, 'COMPRES.Id');/*Valor Null no possible*/ 
    ELSE
      BEGIN
        SELECT *
          INTO vFila
          FROM COMPRES
         WHERE Id = pId;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pId, 'COMPRES.Id');/*Valor d'Id No existeix*/
      END;
    END IF;
    
    IF vFila.DataTancament IS NOT NULL THEN
      RETURN RETORNS(30,pId, 'COMPRES.Id');/*Tancada*/
    END IF;
    /* es comproba data de tancament, per que si es recepcionada ha de ser tancada*/
    IF vFila.DataBaixa IS NOT NULL THEN
      RETURN RETORNS(29,pId, 'COMPRES.Id');/*Tancada*/
    END IF;
    
    BEGIN
      vAux:= NULL;
      SELECT Id
        INTO vAux
        FROM COMPRES_LINIES
       WHERE IdCompra = pId;
      IF vAux IS NOT NULL THEN
            dbms_output.put_line(' ara be vAux ');
            dbms_output.put_line(' vAux ' ||vAux);     
        RETURN RETORNS(32,pId, 'COMPRES.Id');/*Recepcionada*/    
      END IF;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
       null; /*tot ok*/
       WHEN TOO_MANY_ROWS THEN
        RETURN RETORNS(32,pId, 'COMPRES.Id');/*Recepcionada*/    
    END;
    
    DELETE COMPRES WHERE Id = pId;
    
    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha esborrat registre*/
    END IF;
  
  EXCEPTION
    WHEN TE_FILLS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'COMPRES', pId, 'FK Error: '||SQLERRM, 'D', 0);
      RETURN RETORNS(6,pId, 'COMPRES.Id');/*Error general revisar LOG*/
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'COMPRES', pId, SQLERRM, 'D', 0);
      RETURN RETORNS(1,pId, SQLCODE);/*Error general revisar LOG*/
  END;
 
   FUNCTION COMPRES_LINIES_D (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux NUMBER;
    vFila COMPRES%ROWTYPE;
  BEGIN 
 
    IF FUNC_ACCES_PERMES ('COMPRES_LINIES_D', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
    
    IF pId IS NULL THEN 
      RETURN RETORNS(2, 0, 'COMPRES_LINIES.Id');/*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT IdCompra
          INTO vAux
          FROM COMPRES_LINIES
         WHERE Id = pId;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pId, 'COMPRES_LINIES.Id');/*Valor d'Id No existeix*/
      END;
    END IF;
    
    BEGIN
      SELECT *
        INTO vFila
        FROM COMPRES
       WHERE Id = vAux;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pId, 'COMPRES_LINIES.Id');/*Valor d'Id No existeix*/
    END;
    
    IF vFila.DataTancament IS NOT NULL THEN
      RETURN RETORNS(34,pId, 'COMPRES_LINIES.Id');/*Tancada*/
    END IF;

    IF vFila.DataBaixa IS NOT NULL THEN
      RETURN RETORNS(35,pId, 'COMPRES_LINIES.Id');/*donada de baixa*/
    END IF;
    
   
    DELETE COMPRES_LINIES WHERE Id = pId;
    
    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha esborrat registre*/
    END IF;
  
  EXCEPTION
    WHEN TE_FILLS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'COMPRES_LINIES', pId, 'FK Error: '||SQLERRM, 'D', 0);
      RETURN RETORNS(6,pId, 'COMPRES_LINIES.Id');/*Error general revisar LOG*/
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'COMPRES_LINIES', pId, SQLERRM, 'D', 0);
      RETURN RETORNS(1,pId, SQLCODE);/*Error general revisar LOG*/
  END; 
   
  FUNCTION AUTORITZAR_COMPRA (pId NUMBER, pIdAutoritza NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vFila COMPRES%ROWTYPE;
    vAux VARCHAR2 (100);
      
  BEGIN     
 
    IF FUNC_ACCES_PERMES ('AUTORITZAR_COMPRA', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
   
    IF pId IS NOT NULL THEN 
      RETURN RETORNS(2, 0, 'COMPRES.Id');/*Valor Null no possible*/ 
    ELSE
      BEGIN
        SELECT *
          INTO vFila
          FROM COMPRES
         WHERE Id = pId;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pId, 'COMPRES.Id');/*Valor d'Id No existeix*/
      END;
    END IF;
    
    IF vFila.IdAutoritzador IS NOT NULL THEN
      RETURN RETORNS(36,pId, 'COMPRES.Id');/*Tancada*/
    END IF;
    /* es comproba data de tancament, per que si es recepcionada ha de ser tancada*/
    IF vFila.DataBaixa IS NOT NULL THEN
      RETURN RETORNS(37,pId, 'COMPRES.Id');/*Tancada*/
    END IF;
    
    IF pIdAutoritza IS NULL THEN 
      RETURN RETORNS(2, 0, 'COMPRES.IdAutoritzador'); /*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT Id
          INTO vAux
          FROM TREBALLADORS
         WHERE Id = pIdAutoritza;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdAutoritza, 'COMPRES.IdAutoritzador');/*Valor d'Id No existeix*/
      END;
    END IF;
    
    BEGIN  
      SELECT SUM(Preu)
        INTO vRes
        FROM COMPRES_LINIES
        WHERE IdCompra = pId;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pId, 'REPREV.Id');/*Valor d'Id No existeix*/
    END; 
      
    UPDATE COMPRES SET DataTancament = SYSDATE, Preu = vRes, IdAutoritzador = pIdAutoritza WHERE Id = pId;
    
    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
    END IF;

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'AUTORITZAR_COMPRA', vRes, SQLERRM, 'F', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION RECEPCIO (pId NUMBER, pIdReceptor NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vFila COMPRES%ROWTYPE;
    vAux VARCHAR2 (100);
    vRet RETORNS;
      
  BEGIN     
 
    IF FUNC_ACCES_PERMES ('RECEPCIO', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
   
    IF pId IS NOT NULL THEN 
      RETURN RETORNS(2, 0, 'COMPRES.Id');/*Valor Null no possible*/ 
    ELSE
      BEGIN
        SELECT *
          INTO vFila
          FROM COMPRES
         WHERE Id = pId;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pId, 'COMPRES.Id');/*Valor d'Id No existeix*/
      END;
    END IF;
    
    IF vFila.DataTancament IS NULL THEN
      RETURN RETORNS(38,pId, 'COMPRES.Id');/*Tancada*/
    END IF;
    IF vFila.IdReceptor IS NOT NULL THEN
      RETURN RETORNS(39,pId, 'COMPRES.Id');/*Recepcionada*/
    END IF;
    /* es comproba data de tancament, per que si es recepcionada ha de ser tancada*/
    IF vFila.DataBaixa IS NOT NULL THEN
      RETURN RETORNS(40,pId, 'COMPRES.Id');/*Tancada*/
    END IF;
    
    IF pIdReceptor IS NULL THEN 
      RETURN RETORNS(2, 0, 'COMPRES.IdAutoritzador'); /*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT Id
          INTO vAux
          FROM TREBALLADORS
         WHERE Id = pIdReceptor;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdReceptor, 'COMPRES.IdReceptor');/*Valor d'Id No existeix*/
      END;
    END IF;
      
    UPDATE COMPRES SET DataRecepcio = SYSDATE, IdReceptor = pIdReceptor WHERE Id = pId;
    
    IF sql%ROWCOUNT >=1 THEN
      vRet:= PAK_STOCK.ACTUALITZA_STOCK_COMPRA (pId, vFila.IdCentre, vFila.Us, 0);
      IF vRet.IdMissatge = 0 THEN
        RETURN RETORNS(0, pId, null);
      ELSE
        RETURN RETORNS(43, pId, 'COMPRES.I');
      END IF;
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
    END IF;

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'RECEPCIO', vRes, SQLERRM, 'F', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;
  
  FUNCTION REOBRIR_COMPRA (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vFila COMPRES%ROWTYPE;
    vAux VARCHAR2 (100);
      
  BEGIN     
 
    IF FUNC_ACCES_PERMES ('REOBRIR_COMPRA', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
   
    IF pId IS NULL THEN 
      RETURN RETORNS(2, 0, 'COMPRES.Id');/*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT *
          INTO vFila
          FROM COMPRES
         WHERE Id = pId;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pId, 'COMPRES.Id');/*Valor d'Id No existeix*/
      END;
    END IF;
    
    IF vFila.DataTancament IS NULL THEN
      RETURN RETORNS (41,pId,'COMPRES.Id');
    END IF;

    IF vFila.DataRecepcio IS NOT NULL THEN
      RETURN RETORNS (42,pId,'COMPRES.Id');
    END IF;
   
    UPDATE COMPRES SET DataTancament = NULL, IdAutoritzador = NULL, Preu = NULL WHERE Id = pId;
    
    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
    END IF;

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'REOBRIR_COMPRA', vRes, SQLERRM, 'F', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;
  
  FUNCTION BAIXA_COMPRA (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vFila COMPRES%ROWTYPE;
    vAux VARCHAR2 (100);
      
  BEGIN     
 
    IF FUNC_ACCES_PERMES ('BAIXA_COMPRA', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
   
    IF pId IS NULL THEN 
      RETURN RETORNS(2, 0, 'COMPRES.Id');/*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT *
          INTO vFila
          FROM COMPRES
         WHERE Id = pId;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pId, 'COMPRES.Id');/*Valor d'Id No existeix*/
      END;
    END IF;

    IF vFila.DataRecepcio IS NULL THEN
      RETURN RETORNS (31,pId,'COMPRES.Id');
    END IF;
   
    UPDATE COMPRES SET DataBaixa = SYSDATE WHERE Id = pId;
    
    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
    END IF;

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'BAIXA_COMPRA', vRes, SQLERRM, 'F', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;
    

END PAK_COMPRES;
/