create or replace
PACKAGE BODY PAK_DISTRIBUIDORS AS

  TE_FILLS exception;
  pragma exception_init( TE_FILLS, -2292 );

FUNCTION DISTRIBUIDORS_I (pNom VARCHAR2, pNomContacte VARCHAR2, pDocTipus VARCHAR2,
                          pDocNumero VARCHAR2, pAdrea VARCHAR2, pCP VARCHAR2, pCiutat VARCHAR2, pProvincia VARCHAR2,
                          pPais VARCHAR2, pDataBaixa DATE, pIdIdioma NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vAux NUMBER;
  BEGIN

    IF FUNC_ACCES_PERMES ('DISTRIBUIDORS_I', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    IF pNom IS NULL THEN RETURN RETORNS(2, 0, 'DISTRIBUIDORS.Nom');/*Valor Null no possible*/
    ELSIF pDocTipus IS NULL THEN RETURN RETORNS(2, 0, 'DISTRIBUIDORS.DocTipus');/*Valor Null no possible*/
    ELSIF pDocTipus NOT IN ('NIF', 'CIF', 'NIE', 'PASSPORT') THEN RETURN RETORNS(7, 0, 'DISTRIBUIDORS.DocTipus');/*Error Tipus Document*/
    ELSIF pDocNumero IS NULL THEN RETURN RETORNS(2, 0, 'DISTRIBUIDORS.DocNumero');/*Valor Null no possible*/
    END IF;

    IF pIdIdioma IS NULL  THEN
      RETURN RETORNS(2, 0, 'TREBALLADORS.IdIdioma');/*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT Id
          INTO vAux
          FROM IDIOMES
         WHERE Id = pIdIdioma;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdIdioma, 'TREBALLADORS.IdIdioma');/*Valor d'Id No existeix*/
      END;
    END IF;

    INSERT INTO DISTRIBUIDORS (Id, Nom, NomContacte, DocTipus, DocNumero, Adrea, CP, Ciutat, Provincia, Pais, DataBaixa, IdIdioma)
                        VALUES (SEQ_DISTRIBUIDORS.NEXTVAL, pNom, pNomContacte, pDocTipus,
                                pDocNumero, pAdrea, pCP, pCiutat, pProvincia, pPais, pDataBaixa, pIdIdioma)
    RETURNING Id INTO vRes;

    RETURN RETORNS(0, vRes,null);

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'DISTRIBUIDORS', vRes, SQLCODE, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION DISTRIBUIDORS_MARQUES_I (pIdDis NUMBER, pIdMar NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vAux NUMBER;
  BEGIN

    IF FUNC_ACCES_PERMES ('DISTRIBUIDORS_MARQUES_I', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    IF pIdDis IS NULL THEN
      RETURN RETORNS(2, 0, 'DISTRIBUIDORS_MARQUES.IdDistribuidor');  /*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT Id
          INTO vAux
          FROM DISTRIBUIDORS
         WHERE Id = pIdDis;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3, pIdDis, 'DISTRIBUIDORS_MARQUES.IdDistribuidor');/*Valor d'Id No existeix*/
      END;
    END IF;

    IF pIdMar IS NULL THEN
      RETURN RETORNS(2, 0, 'DISTRIBUIDORS_MARQUES.IdMarca');/*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT Id
          INTO vAux
          FROM MARQUES
         WHERE Id = pIdMar;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdMar, 'DISTRIBUIDORS_MARQUES.IdMarca');/*Valor d'Id No existeix*/
      END;
    END IF;

    BEGIN
      SELECT 1
        INTO vAux
        FROM DISTRIBUIDORS_MARQUES
       WHERE IdDistribuidor = pIdDis AND
             IdMarca = pIdMar;
      IF vAux = 1 THEN
        RETURN RETORNS(14, 0, 'DISTRIBUIDORS_MARQUES');
      END IF;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
       null;/*tot OK*/
    END;

    INSERT INTO DISTRIBUIDORS_MARQUES VALUES (SEQ_DISTRIBUIDORS_MARQUES.NEXTVAL, pIdDis, pIdMar)
    RETURNING Id INTO VRes;

    RETURN RETORNS(0, vRes, null);

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'DISTRIBUIDORS_MARQUES', vRes, SQLERRM, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

FUNCTION DISTRIBUIDORS_CONTACTE_I (pIdDis NUMBER, pTipus VARCHAR2, pDada VARCHAR2, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vAux NUMBER;
  BEGIN

    IF FUNC_ACCES_PERMES ('DISTRIBUIDORS_CONTACTE_I', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    IF pIdDis IS NULL THEN RETURN RETORNS(2, 0, 'DISTRIBUIDORS_CONTACTE.IdDistribuidor');/*Valor Null no possible*/
    ELSIF pTipus IS NULL THEN RETURN RETORNS(2, 0, 'DISTRIBUIDORS_CONTACTE.Tipus');/*Valor Null no possible*/
    ELSIF pTipus NOT IN ('TP', 'TT', 'TM', 'FA', 'EP', 'ET') THEN RETURN RETORNS(9, 0, 'DISTRIBUIDORS_CONTACTE.Tipus');/*Error Tipus Contacte*/
    ELSIF pDada IS NULL THEN RETURN RETORNS(2, 0, 'DISTRIBUIDORS_CONTACTE.Dada');/*Valor Null no possible*/
    END IF;

    BEGIN
      SELECT Id
        INTO vAux
        FROM DISTRIBUIDORS
       WHERE Id = pIdDis;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pIdDis, 'DISTRIBUIDORS_CONTACTE.IdDistribuidor');/*Valor d'Id No existeix*/
    END;

    INSERT INTO DISTRIBUIDORS_CONTACTE VALUES (SEQ_DISTRIBUIDORS_CONTACTE.NEXTVAL, pIdDis, pTipus, pDada)
    RETURNING Id INTO VRes;

    RETURN RETORNS(0, vRes, null);

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'DISTRIBUIDORS_CONTACTE', vRes, SQLCODE, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION DISTRIBUIDORS_U (pId NUMBER, pNom VARCHAR2, pNomContacte VARCHAR2, pDocTipus VARCHAR2,
                            pDocNumero VARCHAR2, pAdrea VARCHAR2, pCP VARCHAR2, pCiutat VARCHAR2, pProvincia VARCHAR2,
                            pPais VARCHAR2, pDataBaixa DATE, pIdIdioma NUMBER, pIdUser NUMBER) RETURN RETORNS  IS
    vAux VARCHAR2 (100);
    vModif BOOLEAN := FALSE;
    vFila DISTRIBUIDORS%ROWTYPE;
    vSQL VARCHAR2(1000):='';
  BEGIN

    IF FUNC_ACCES_PERMES ('DISTRIBUIDORS_U', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    IF pDocTipus IS NOT NULL AND pDocTipus NOT IN ('NIF', 'CIF', 'NIE', 'PASSPORT') THEN
      RETURN RETORNS(7, 0, 'DISTRIBUIDORS.DocTipus');
    END IF;

  IF pId IS NULL THEN
     RETURN RETORNS(2, 0, 'DISTRIBUIDORS.Id');/*Valor Null no possible*/
   ELSE
     BEGIN
       SELECT *
         INTO vFila
         FROM DISTRIBUIDORS
        WHERE Id = pId;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
         RETURN RETORNS(3,pId, 'DISTRIBUIDORS.Id');/*Valor d'Id No existeix*/
     END;
   END IF;

    IF pNom IS NOT NULL AND pNom <> vFila.Nom THEN
      vSQL := vSQL||' Nom = ''' || pNom ||''',';
      vModif := TRUE;
    END IF;

    IF pNomContacte IS NOT NULL AND pNomContacte <> vFila.NomContacte THEN
      vSQL := vSQL||' NomContacte = ''' || pNomContacte ||''',';
      vModif := TRUE;
    END IF;

    IF pDocTipus IS NOT NULL AND pDocTipus <> vFila.DocTipus THEN
      vSQL := vSQL||' DocTipus = ''' || pDocTipus ||''',';
      vModif := TRUE;
    END IF;

    IF pDocNumero IS NOT NULL AND pDocNumero <> vFila.DocNumero THEN
      vSQL := vSQL||' DocNumero = ''' || pDocNumero ||''',';
      vModif := TRUE;
    END IF;

    IF pAdrea IS NOT NULL AND pAdrea <> vFila.Adrea THEN
      vSQL := vSQL||' Adrea = ''' || pAdrea ||''',';
      vModif := TRUE;
    END IF;

    IF pCP IS NOT NULL AND pCP <> vFila.CP THEN
      vSQL := vSQL||' CP = ''' || pCP ||''',';
      vModif := TRUE;
    END IF;

    IF pCiutat IS NOT NULL AND pCiutat <> vFila.Ciutat THEN
      vSQL := vSQL||' Ciutat = ''' || pCiutat ||''',';
      vModif := TRUE;
    END IF;

    IF pProvincia IS NOT NULL AND pProvincia <> vFila.Provincia THEN
      vSQL := vSQL||' Provincia = ''' || pProvincia ||''',';
      vModif := TRUE;
    END IF;

    IF pPais IS NOT NULL AND pPais <> vFila.Pais THEN
      vSQL := vSQL||' Pais = ''' || pPais ||''',';
      vModif := TRUE;
    END IF;

    IF pDataBaixa IS NOT NULL AND pDataBaixa <> vFila.DataBaixa THEN
      vSQL := vSQL||' DataBaixa = ''' || pDataBaixa ||''',';
      vModif := TRUE;
    END IF;

    IF pIdIdioma IS NOT NULL AND pIdIdioma <> vFila.IdIdioma THEN
      BEGIN
        SELECT Id
          INTO vAux
          FROM IDIOMES
         WHERE Id = pIdIdioma;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdIdioma, 'DISTRIBUIDORS.IdIdioma');/*Valor d'Id No existeix*/
      END;
      vSQL := vSQL||' IdIdioma = ' || pIdIdioma||',';
      vModif := TRUE;
    END IF;

    IF vModif THEN
      vSQL := 'UPDATE DISTRIBUIDORS SET ' || SUBSTR(vSQL, 1,LENGTH (vSQL)-1)|| ' WHERE Id = '|| pId;
      EXECUTE IMMEDIATE (vSQL);

      IF sql%ROWCOUNT >=1 THEN
        RETURN RETORNS(0, pId, null);
      ELSE
        RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
      END IF;
    ELSE
      RETURN RETORNS(4, pId, null);/*No s'ha realitzat cap acci*/
    END IF;
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'DISTRIBUIDORS', pId, vSQL, 'U', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

 FUNCTION DISTRIBUIDORS_MARQUES_U (pId NUMBER, pIdDis NUMBER, pIdMar NUMBER, pIdUser NUMBER) RETURN RETORNS IS

  BEGIN

    IF FUNC_ACCES_PERMES ('DISTRIBUIDORS_MARQUES_U', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    RETURN RETORNS (15, 0, 'DISTRIBUIDORS_MARQUES_U');

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'DISTRIBUIDORS_MARQUES_U', pId, SQLERRM, 'U', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION DISTRIBUIDORS_CONTACTE_U (pId NUMBER, pIdDis NUMBER, pTipus VARCHAR2, pDada VARCHAR2, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
    vModif BOOLEAN := FALSE;
    vFila DISTRIBUIDORS_CONTACTE%ROWTYPE;
    vSQL VARCHAR2(1000):='';
  BEGIN

    IF FUNC_ACCES_PERMES ('DISTRIBUIDORS_CONTACTE_U', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    IF pTipus IS NOT NULL AND pTipus NOT IN ('TP', 'TT', 'TM', 'FA', 'EP', 'ET') THEN
      RETURN RETORNS(9, 0, 'DISTRIBUIDORS_CONTACTE.Tipus');/*Error Tipus Contacte*/
    END IF;


   IF pId IS NULL THEN
     RETURN RETORNS(2, 0, 'DISTRIBUIDORS_CONTACTE.Id');/*Valor Null no possible*/
   ELSE
     BEGIN
       SELECT *
         INTO vFila
         FROM DISTRIBUIDORS_CONTACTE
        WHERE Id = pId;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
         RETURN RETORNS(3,pId, 'DISTRIBUIDORS_CONTACTE.Id');/*Valor d'Id No existeix*/
     END;
   END IF;

    IF pIdDis IS NOT NULL AND pIdDis <> vFila.IdDistribuidor THEN
      BEGIN
        SELECT Id
          INTO vAux
          FROM DISTRIBUIDORS
         WHERE Id = pIdDis;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdDis, 'DISTRIBUIDORS_CONTACTE.IdDistribuidor');/*Valor d'Id No existeix*/
      END;
      vSQL := vSQL||' IdDistribuidor = ' || pIdDis||',';
      vModif := TRUE;
    END IF;

    IF pTipus IS NOT NULL AND pTipus <> vFila.Tipus THEN
      vSQL := vSQL||' Tipus = ''' || pTipus ||''',';
      vModif := TRUE;
    END IF;


    IF pDada <> vFila.Dada THEN
      vSQL := vSQL||' Dada = ''' || pDada||''',';
      vModif := TRUE;
    END IF;

    IF vModif THEN
      vSQL := 'UPDATE DISTRIBUIDORS_CONTACTE SET ' || SUBSTR(vSQL, 1,LENGTH (vSQL)-1)|| ' WHERE Id = '|| pId;
      EXECUTE IMMEDIATE (vSQL);

      IF sql%ROWCOUNT >=1 THEN
        RETURN RETORNS(0, pId, null);
      ELSE
        RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
      END IF;
    ELSE
      RETURN RETORNS(4, pId, null);/*No s'ha realitzat cap acci*/
    END IF;
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'DISTRIBUIDORS_CONTACTE', pId, vSQL, 'U', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

FUNCTION DISTRIBUIDORS_D (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
  BEGIN

    IF FUNC_ACCES_PERMES ('DISTRIBUIDORS_D', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    BEGIN
      SELECT Id
        INTO vAux
        FROM DISTRIBUIDORS
       WHERE Id = pId;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pId, 'DISTRIBUIDORS.Id');/*Valor d'Id No existeix*/
    END;

    DELETE DISTRIBUIDORS WHERE Id = pId;

    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha esborrat registre*/
    END IF;

  EXCEPTION
    WHEN TE_FILLS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'DISTRIBUIDORS', pId, 'FK Error: '||SQLERRM, 'D', 0);
      RETURN RETORNS(6,pId, 'DISTRIBUIDORS.Id');/*Error general revisar LOG*/
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'DISTRIBUIDORS', pId, SQLERRM, 'D', 0);
      RETURN RETORNS(1,pId, SQLCODE);/*Error general revisar LOG*/
  END;

FUNCTION DISTRIBUIDORS_MARQUES_D (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
  BEGIN

    IF FUNC_ACCES_PERMES ('DISTRIBUIDORS_MARQUES_D', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    BEGIN
      SELECT Id
        INTO vAux
        FROM DISTRIBUIDORS_MARQUES
       WHERE Id = pId;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pId, 'DISTRIBUIDORS_MARQUES.Id');/*Valor d'Id No existeix*/
    END;

    DELETE DISTRIBUIDORS_MARQUES WHERE Id = pId;

    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha esborrat registre*/
    END IF;

  EXCEPTION
    WHEN TE_FILLS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'DISTRIBUIDORS_MARQUES', pId, 'FK Error: '||SQLERRM, 'D', 0);
      RETURN RETORNS(6,pId, 'DISTRIBUIDORS_MARQUES.Id');/*Error general revisar LOG*/
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'DISTRIBUIDORS_MARQUES', pId, SQLERRM, 'D', 0);
      RETURN RETORNS(1,pId, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION DISTRIBUIDORS_CONTACTE_D (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
  BEGIN

    IF FUNC_ACCES_PERMES ('DISTRIBUIDORS_CONTACTE_D', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    IF pId IS NULL THEN
      RETURN RETORNS(2, 0, 'DISTRIBUIDORS_CONTACTE.Id');/*Valor Null no possible*/
    END IF;

    BEGIN
      SELECT Id
        INTO vAux
        FROM DISTRIBUIDORS_CONTACTE
       WHERE Id = pId;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pId, 'DISTRIBUIDORS_CONTACTE.pId');/*Valor d'Id No existeix*/
    END;

    DELETE DISTRIBUIDORS_CONTACTE WHERE Id = pId;

    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha esborrat registre*/
    END IF;

  EXCEPTION
   WHEN TE_FILLS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'DISTRIBUIDORS_CONTACTE', pId, 'FK Error: '||SQLERRM, 'D', 0);
      RETURN RETORNS(6,pId, 'DISTRIBUIDORS_CONTACTE.Id');/*Error general revisar LOG*/
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'DISTRIBUIDORS_CONTACTE', pId, SQLERRM, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

END PAK_DISTRIBUIDORS;