create or replace
PACKAGE BODY PAK_STOCK AS

  TE_FILLS exception;
  pragma exception_init( TE_FILLS, -2292 );
  vMarge_Comercial NUMBER := 1.30;
    
  FUNCTION TREU_DE_STOCK (pIdVeh NUMBER, pIdCentre NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux NUMBER;
  BEGIN
    IF FUNC_ACCES_PERMES ('TREU_DE_STOCK', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;
    BEGIN
      SELECT Id
        INTO vAux
        FROM STOCK
       WHERE idVehicle = pIdVeh AND
             idCentre = pIdCentre AND
             DataSortida IS NULL;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pIdVeh, 'STOCK.IdVehicle');/*Valor d'Id No existeix*/
    END;  
    UPDATE STOCK SET datasortida = SYSDATE WHERE Id = vAux;
    IF sql%ROWCOUNT <>1 THEN
      RETURN RETORNS(5, pIdVeh, null); /*No s'ha esborrat registre*/
    END IF;

    RETURN RETORNS(0, pIdVeh, null);
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'TREU_DE_STOCK', pIdVeh, SQLERRM, 'F', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION TORNA_A_STOCK (pIdVeh NUMBER, pIdCentre NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux NUMBER;
  BEGIN
   
    IF FUNC_ACCES_PERMES ('TORNA_A_STOCK', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;
    BEGIN
      SELECT Id
        INTO vAux
        FROM (SELECT Id, dataSortida
                FROM STOCK
              WHERE IdVehicle = pIdVeh AND
                    IdCentre = pIdCentre
           ORDER BY DataSortida DESC)
       WHERE ROWNUM =1;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pIdVeh, 'STOCK.IdVehicle');/*Valor d'Id No existeix*/
    END;    
    UPDATE STOCK SET DataSortida = NULL WHERE Id = vAux;
    IF sql%ROWCOUNT <>1 THEN
      RETURN RETORNS(5, pIdVeh, null); /*No s'ha esborrat registre*/
    END IF;

    RETURN RETORNS(0, pIdVeh, null);
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'TORNA_A_STOCK', pIdVeh, SQLERRM, 'F', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;
  
  FUNCTION ACTUALITZA_STOCK_COMPRA (pIdCompra NUMBER, pIdCentre NUMBER, pUs VARCHAR2, pIdUser NUMBER) RETURN RETORNS IS
    
    CURSOR cVehicles IS
      SELECT IdVehicle, Preu*vMarge_Comercial
      FROM COMPRES_LINIES
      WHERE IdCompra = pIdCompra;
   
    vIdVehicle VEHICLES.Id%TYPE;        
    vPreu STOCK.PreuVenta%TYPE;
    
  BEGIN
  
    IF FUNC_ACCES_PERMES ('ACTUALITZA_STOCK_COMPRA', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;
    
    OPEN cVehicles;
      
      FETCH cVehicles INTO vIdVehicle, vPreu;
     
      INSERT INTO STOCK (Id, IdVehicle, IdCentre, DataEntrada, PreuVenta, Us) 
                 VALUES (SEQ_STOCK.NEXTVAL, vIdVehicle, pIdCentre, SYSDATE, vPreu, pUs);

      IF sql%ROWCOUNT <>1 THEN
        RETURN RETORNS(5, pIdCompra, null); /*No s'ha esborrat registre*/
      END IF;
  
    CLOSE cVehicles;
  RETURN RETORNS(0, pIdCompra, null);
  EXCEPTION
    WHEN OTHERS THEN
      IF cVehicles%ISOPEN THEN
        CLOSE cVehicles;
      END IF;
      PROC_INSERT_LOGERROR(SQLCODE, 'ACTUALIZA_STOCK', pIdCompra, SQLERRM, 'F', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION RESERVA_VEHICLE (pIdVeh NUMBER, pIdCentre NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux NUMBER;
  BEGIN
  
    IF FUNC_ACCES_PERMES ('RESERVA_VEHICLE', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;
    BEGIN
      SELECT Id
      INTO vAux
      FROM STOCK
      WHERE idVehicle = pIdVeh AND
            idCentre = pIdCentre AND
            DataEncarrec IS NULL;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pIdVeh, 'STOCK.IdVehicle');/*Valor d'Id No existeix*/
    END;   
    
    UPDATE STOCK SET DataEncarrec = SYSDATE WHERE Id = vAux;
    IF sql%ROWCOUNT <>1 THEN
      RETURN RETORNS(5, pIdVeh, null); /*No s'ha esborrat registre*/
    END IF;

    RETURN RETORNS(0, pIdVeh, null);
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'RESERVA_VEHICLE', pIdVeh, SQLERRM, 'F', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION DESRESERVA_VEHICLE (pIdVeh NUMBER, pIdCentre NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux NUMBER;
  BEGIN
   
    IF FUNC_ACCES_PERMES ('DESRESERVA_VEHICLE', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;
  
    BEGIN
      SELECT Id
        INTO vAux
        FROM STOCK
       WHERE IdVehicle = pIdVeh AND
             IdCentre = pIdCentre AND
             DataEncarrec IS NOT NULL AND
             DataSortida IS NULL;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pIdVeh, 'STOCK.IdVehicle');/*Valor d'Id No existeix*/
    END;     
    
    UPDATE STOCK SET DataEncarrec = NULL WHERE Id = vAux;
    IF sql%ROWCOUNT <>1 THEN
      RETURN RETORNS(5, pIdVeh, null); /*No s'ha esborrat registre*/
    END IF;

    RETURN RETORNS(0, pIdVeh, null);
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'TORNA_A_STOCK', pIdVeh, SQLERRM, 'F', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;
  
  
END PAK_STOCK;
/