create or replace
PACKAGE BODY PAK_TREBALLADORS AS

  TE_FILLS exception;
  pragma exception_init( TE_FILLS, -2292 );

FUNCTION TREBALLADORS_I (pNom VARCHAR2, pCognom1 VARCHAR2, pCognom2 VARCHAR2, pRol VARCHAR2, pNomUsuari VARCHAR2, 
                         pContrasenya VARCHAR2, pIdCentre NUMBER, pDataBaixa DATE, pIdIdioma NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vAux NUMBER;
  BEGIN 

    IF FUNC_ACCES_PERMES ('TREBALLADORS_I', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
    
    IF pNom IS NULL THEN RETURN RETORNS(2, 0, 'TREBALLADORS.Nom');/*Valor Null no possible*/
    ELSIF pCognom1 IS NULL THEN RETURN RETORNS(2, 0, 'TREBALLADORS.Cognom1');/*Valor Null no possible*/
    ELSIF pRol IS NULL THEN RETURN RETORNS(2, 0, 'TREBALLADORS.Rol');/*Valor Null no possible*/
    ELSIF pRol NOT IN ('AD', 'RE', 'EN', 'VE', 'ME') THEN RETURN RETORNS(12, 0, 'TREBALLADORS.Rol');/*Error Tipus Treballador*/         
    ELSIF pNomUsuari IS NULL THEN RETURN RETORNS(2, 0, 'TREBALLADORS.NomUsuari');/*Valor Null no possible*/ 
    ELSIF pContrasenya IS NULL THEN RETURN RETORNS(2, 0, 'TREBALLADORS.Contrasenya');/*Valor Null no possible*/ 
    END IF;

    IF pIdCentre IS NULL THEN 
      RETURN RETORNS(2, 0, 'TREBALLADORS.IdCentre');/*Valor Null no possible*/ 
    ELSE
      BEGIN
        SELECT Id
          INTO vAux
          FROM CENTRES
         WHERE Id = pIdCentre;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdCentre, 'TREBALLADORS.IdCentre');/*Valor d'Id No existeix*/
      END;
    END IF;

    IF pIdIdioma IS NULL  THEN
      RETURN RETORNS(2, 0, 'TREBALLADORS.IdIdioma');/*Valor Null no possible*/ 
    ELSE
      BEGIN
        SELECT Id
          INTO vAux
          FROM IDIOMES
         WHERE Id = pIdIdioma;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdIdioma, 'TREBALLADORS.IdIdioma');/*Valor d'Id No existeix*/
      END;
    END IF;

    INSERT INTO TREBALLADORS (Id, Nom, Cognom1, Cognom2, Rol, NomUsuari, Contrasenya, DataBaixa, IdCentre, IdIdioma)
                        VALUES (SEQ_TREBALLADORS.NEXTVAL, pNom, pCognom1, pCognom2, pRol, pNomUsuari, pContrasenya, pDataBaixa, pIdCentre, pIdIdioma)
    RETURNING Id INTO vRes;

    RETURN RETORNS(0, vRes,null);
 
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'TREBALLADORS', vRes, SQLCODE, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

FUNCTION TREBALLADORS_CONTACTE_I (pIdTre NUMBER, pTipus VARCHAR2, pDada VARCHAR2, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vAux NUMBER;
  BEGIN 

    IF FUNC_ACCES_PERMES ('TREBALLADORS_CONTACTE_I', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
    
    IF pIdTre IS NULL THEN RETURN RETORNS(2, 0, 'TREBALLADORS_CONTACTE.IdTreballador');/*Valor Null no possible*/
    ELSIF pTipus IS NULL THEN RETURN RETORNS(2, 0, 'TREBALLADORS_CONTACTE.Tipus');/*Valor Null no possible*/
    ELSIF pTipus NOT IN ('TP', 'TT', 'TM', 'FA', 'EP', 'ET') THEN RETURN RETORNS(9, 0, 'TREBALLADORS_CONTACTE.Tipus');/*Error Tipus Contacte*/     
    ELSIF pDada IS NULL THEN RETURN RETORNS(2, 0, 'TREBALLADORS_CONTACTE.Dada');/*Valor Null no possible*/
    END IF;

    BEGIN
      SELECT Id
        INTO vAux
        FROM TREBALLADORS
       WHERE Id = pIdTre;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pIdTre, 'TREBALLADORS_CONTACTE.IdTreballador');/*Valor d'Id No existeix*/
    END;

    INSERT INTO TREBALLADORS_CONTACTE VALUES (SEQ_TREBALLADORS_CONTACTE.NEXTVAL, pIdTre, pTipus, pDada)
    RETURNING Id INTO VRes;

    RETURN RETORNS(0, vRes, null);

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'TREBALLADORS_CONTACTE', vRes, SQLCODE, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION TREBALLADORS_U (pId NUMBER, pNom VARCHAR2, pCognom1 VARCHAR2, pCognom2 VARCHAR2, pRol VARCHAR2, pNomUsuari VARCHAR2, 
                         pContrasenya VARCHAR2, pIdCentre NUMBER, pDataBaixa DATE, pIdIdioma NUMBER, pIdUser NUMBER) RETURN RETORNS  IS
    vAux VARCHAR2 (100);
    vModif BOOLEAN := FALSE;
    vFila TREBALLADORS%ROWTYPE;
    vSQL VARCHAR2(1000):='';
  BEGIN 

    IF FUNC_ACCES_PERMES ('TREBALLADORS_U', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
    
    IF pRol IS NOT NULL AND pRol NOT IN ('AD', 'RE', 'EN', 'VE', 'ME') THEN 
      RETURN RETORNS(12, 0, 'TREBALLADORS.Rol');
    END IF;
    
  IF pId IS NULL THEN 
     RETURN RETORNS(2, 0, 'TREBALLADORS.Id');/*Valor Null no possible*/
   ELSE
     BEGIN
       SELECT *
         INTO vFila
         FROM TREBALLADORS
        WHERE Id = pId;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
         RETURN RETORNS(3,pId, 'TREBALLADORS.Id');/*Valor d'Id No existeix*/
     END;
   END IF;

    IF pNom IS NOT NULL AND pNom <> vFila.Nom THEN
      vSQL := vSQL||' Nom = ''' || pNom ||''',';
      vModif := TRUE;
    END IF;

    IF pCognom1 IS NOT NULL AND pCognom1 <> vFila.Cognom1 THEN
      vSQL := vSQL||' Cognom1 = ''' || pCognom1 ||''',';
      vModif := TRUE;
    END IF;
    
    IF pCognom2 IS NOT NULL AND pCognom2 <> vFila.Cognom2 THEN
      vSQL := vSQL||' Cognom2 = ''' || pCognom2 ||''',';
      vModif := TRUE;
    END IF;

    IF pRol IS NOT NULL AND pRol <> vFila.Rol THEN
      vSQL := vSQL||' Rol = ''' || pRol ||''',';
      vModif := TRUE;
    END IF;
    
    IF pNomUsuari IS NOT NULL AND pNomUsuari <> vFila.NomUsuari THEN
      vSQL := vSQL||' NomUsuari = ''' || pNomUsuari ||''',';
      vModif := TRUE;
    END IF;   
    
    IF pContrasenya IS NOT NULL AND pContrasenya <> vFila.Contrasenya THEN
      vSQL := vSQL||' Contrasenya = ''' || pContrasenya ||''',';
      vModif := TRUE;
    END IF;  
    
    IF pDataBaixa IS NOT NULL AND pDataBaixa <> vFila.DataBaixa THEN
      vSQL := vSQL||' DataBaixa = ''' || pDataBaixa ||''',';
      vModif := TRUE;
    END IF; 
    
    IF pIdIdioma IS NOT NULL AND pIdIdioma <> vFila.IdIdioma THEN
      BEGIN
        SELECT Id
          INTO vAux
          FROM IDIOMES
         WHERE Id = pIdIdioma;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdIdioma, 'TREBALLADORS.IdIdioma');/*Valor d'Id No existeix*/
      END;
      vSQL := vSQL||' IdIdioma = ' || pIdIdioma||',';
      vModif := TRUE;
    END IF;
    
    IF pIdCentre IS NOT NULL AND pIdCentre <> vFila.IdCentre THEN
      BEGIN
        SELECT Id
          INTO vAux
          FROM CENTRES
         WHERE Id = pIdCentre;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdIdioma, 'TREBALLADORS.IdCentre');/*Valor d'Id No existeix*/
      END;
      vSQL := vSQL||' IdCentre = ' || pIdCentre||',';
      vModif := TRUE;
    END IF;
  
    IF vModif THEN
      vSQL := 'UPDATE TREBALLADORS SET ' || SUBSTR(vSQL, 1,LENGTH (vSQL)-1)|| ' WHERE Id = '|| pId;
      EXECUTE IMMEDIATE (vSQL);

      IF sql%ROWCOUNT >=1 THEN
        RETURN RETORNS(0, pId, null);
      ELSE
        RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
      END IF;
    ELSE
      RETURN RETORNS(4, pId, null);/*No s'ha realitzat cap acci*/
    END IF;
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'TREBALLADORS', pId, vSQL, 'U', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION TREBALLADORS_CONTACTE_U (pId NUMBER, pIdTre NUMBER, pTipus VARCHAR2, pDada VARCHAR2, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
    vModif BOOLEAN := FALSE;
    vFila TREBALLADORS_CONTACTE%ROWTYPE;
    vSQL VARCHAR2(1000):='';
  BEGIN 

    IF FUNC_ACCES_PERMES ('TREBALLADORS_CONTACTE_U', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 

    IF pTipus IS NOT NULL AND pTipus NOT IN ('TP', 'TT', 'TM', 'FA', 'EP', 'ET') THEN 
      RETURN RETORNS(9, 0, 'TREBALLADORS_CONTACTE.Tipus');/*Error Tipus Contacte*/     
    END IF;

    
   IF pId IS NULL THEN 
     RETURN RETORNS(2, 0, 'TREBALLADORS_CONTACTE.Id');/*Valor Null no possible*/
   ELSE
     BEGIN
       SELECT *
         INTO vFila
         FROM TREBALLADORS_CONTACTE
        WHERE Id = pId;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
         RETURN RETORNS(3,pId, 'TREBALLADORS_CONTACTE.Id');/*Valor d'Id No existeix*/
     END;
   END IF;
   
    IF pIdTre IS NOT NULL AND pIdTre <> vFila.IdTreballador THEN
      BEGIN
        SELECT Id
          INTO vAux
          FROM TREBALLADORS
         WHERE Id = pIdTre;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdTre, 'TREBALLADORS_CONTACTE.IdTreballador');/*Valor d'Id No existeix*/
      END;
      vSQL := vSQL||' IdTreballador = ' || pIdTre||',';
      vModif := TRUE;
    END IF;
  
    IF pTipus IS NOT NULL AND pTipus <> vFila.Tipus THEN
      vSQL := vSQL||' Tipus = ''' || pTipus ||''',';
      vModif := TRUE;
    END IF;


    IF pDada <> vFila.Dada THEN
      vSQL := vSQL||' Dada = ''' || pDada||''',';
      vModif := TRUE;
    END IF;

    IF vModif THEN
      vSQL := 'UPDATE TREBALLADORS_CONTACTE SET ' || SUBSTR(vSQL, 1,LENGTH (vSQL)-1)|| ' WHERE Id = '|| pId;
      EXECUTE IMMEDIATE (vSQL);

      IF sql%ROWCOUNT >=1 THEN
        RETURN RETORNS(0, pId, null);
      ELSE
        RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
      END IF;
    ELSE
      RETURN RETORNS(4, pId, null);/*No s'ha realitzat cap acci*/
    END IF;
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'TREBALLADORS_CONTACTE', pId, vSQL, 'U', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

FUNCTION TREBALLADORS_D (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
  BEGIN 
 
    IF FUNC_ACCES_PERMES ('TREBALLADORS_D', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
     
    BEGIN
      SELECT Id
        INTO vAux
        FROM TREBALLADORS
       WHERE Id = pId;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pId, 'TREBALLADORS.Id');/*Valor d'Id No existeix*/
    END;
    
    DELETE TREBALLADORS WHERE Id = pId;
    
    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha esborrat registre*/
    END IF;
  
  EXCEPTION
    WHEN TE_FILLS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'TREBALLADORS', pId, 'FK Error: '||SQLERRM, 'D', 0);
      RETURN RETORNS(6,pId, 'TREBALLADORS.Id');/*Error general revisar LOG*/
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'TREBALLADORS', pId, SQLERRM, 'D', 0);
      RETURN RETORNS(1,pId, SQLCODE);/*Error general revisar LOG*/
  END;
  
  FUNCTION TREBALLADORS_CONTACTE_D (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
  BEGIN 
 
    IF FUNC_ACCES_PERMES ('TREBALLADORS_CONTACTE_D', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
     
    IF pId IS NULL THEN
      RETURN RETORNS(2, 0, 'TREBALLADORS_CONTACTE.Id');/*Valor Null no possible*/
    END IF;
  
    BEGIN
      SELECT Id
        INTO vAux
        FROM TREBALLADORS_CONTACTE
       WHERE Id = pId;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pId, 'TREBALLADORS_CONTACTE.pId');/*Valor d'Id No existeix*/
    END;
  
    DELETE TREBALLADORS_CONTACTE WHERE Id = pId;
    
    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha esborrat registre*/
    END IF;
  
  EXCEPTION
   WHEN TE_FILLS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'TREBALLADORS_CONTACTE', pId, 'FK Error: '||SQLERRM, 'D', 0);
      RETURN RETORNS(6,pId, 'TREBALLADORS_CONTACTE.Id');/*Error general revisar LOG*/  
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'TREBALLADORS_CONTACTE', pId, SQLERRM, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;
  
  FUNCTION CANVI_CENTRE (pId NUMBER, pIdCentre NUMBER, pIdUser NUMBER) RETURN RETORNS IS
     vAux VARCHAR2 (100);
  BEGIN 
  
    IF FUNC_ACCES_PERMES ('CANVI_CENTRE', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
     
    IF pId IS NULL THEN
      RETURN RETORNS(2, 0, 'TREBALLADORS.Id');/*Valor Null no possible*/
    ELSE    
      BEGIN
        SELECT Id
          INTO vAux
          FROM TREBALLADORS
         WHERE Id = pId;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pId, 'TREBALLADORS.Id');/*Valor d'Id No existeix*/
      END;
    END IF;
    
    IF pIdCentre IS NULL THEN
      RETURN RETORNS(2, 0, 'CENTRES.Id');/*Valor Null no possible*/
    ELSE    
      BEGIN
        SELECT Id
          INTO vAux
          FROM CENTRES
         WHERE Id = pIdCentre;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdCentre, 'CENTRES.Id');/*Valor d'Id No existeix*/
      END;
    END IF;
    
    UPDATE TREBALLADORS SET IdCentre = pIdCentre WHERE Id = pId;

    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
    END IF;

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CANVI_CENTRE', pId, SQLERRM, 'F', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;
  
  FUNCTION BAIXA_TREBALLADOR (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
     vAux VARCHAR2 (100);
  BEGIN 
  
    IF FUNC_ACCES_PERMES ('BAIXA_TREBALLADOR', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
     
    IF pId IS NULL THEN
      RETURN RETORNS(2, 0, 'TREBALLADORS.Id');/*Valor Null no possible*/
    ELSE    
      BEGIN
        SELECT Id
          INTO vAux
          FROM TREBALLADORS
         WHERE Id = pId;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pId, 'TREBALLADORS.Id');/*Valor d'Id No existeix*/
      END;
    END IF;
    
    UPDATE TREBALLADORS SET DataBaixa = SYSDATE WHERE Id = pId;

    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
    END IF;

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'BAIXA_TREBALLADOR', pId, SQLERRM, 'F', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END; 
    
END PAK_TREBALLADORS;
/