create or replace
PACKAGE BODY PAK_VEHICLES AS

  TE_FILLS exception;
  pragma exception_init( TE_FILLS, -2292 );

FUNCTION VEHICLES_I (pIdCon NUMBER, pIdCli NUMBER, pBastidor VARCHAR2, pMatricula VARCHAR2,
                    pKm NUMBER, pDataRevisio DATE, pDataBaixa DATE, pAnyFabricacio NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vAux NUMBER;
  BEGIN

    IF FUNC_ACCES_PERMES ('VEHICLES_I', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    IF pAnyFabricacio IS NOT NULL AND (pAnyFabricacio > 2100 OR pAnyFabricacio < 1900)
               THEN RETURN RETORNS(8, 0, 'VEHICLES.AnyFabricacio');/*Any Incorrecte*/
    END IF;

    IF pIdCon IS NULL THEN
      RETURN RETORNS(2, 0, 'VEHICLES.IdConfigFin');
    ELSE
      BEGIN
        SELECT Id
          INTO vAux
          FROM CONFIGFIN
         WHERE Id = pIdCon;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdCon, 'VEHICLES.IdConfigFin');/*Valor d'Id No existeix*/
      END;
    END IF;

    IF pIdCli IS NOT NULL THEN
      BEGIN
        SELECT Id
          INTO vAux
          FROM CLIENTS
         WHERE Id = pIdCli;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdCLi, 'VEHICLES.IdClient');/*Valor d'Id No existeix*/
      END;
    END IF;

    INSERT INTO VEHICLES (Id, IdConfigFin, IdClient, Bastidor, Matricula, Km, DataRevisio, DataBaixa, AnyFabricacio)
                        VALUES (SEQ_VEHICLES.NEXTVAL, pIdCon, pIdCli, pBastidor, pMatricula, pKm, pDataRevisio, pDataBaixa, pAnyFabricacio)
    RETURNING Id INTO vRes;

    RETURN RETORNS(0, vRes,null);

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VEHICLES', vRes, SQLCODE, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

FUNCTION VEHICLES_EXTRES_I (pIdVeh NUMBER, pIdExt NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vAux NUMBER;
  BEGIN

    IF FUNC_ACCES_PERMES ('VEHICLES_EXTRES_I', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    IF pIdVeh IS NULL THEN
      RETURN RETORNS(2, 0, 'VEHICLES_EXTRES.IdVehicle');  /*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT Id
          INTO vAux
          FROM VEHICLES
         WHERE Id = pIdVeh;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3, pIdVeh, 'VEHICLES_EXTRES.IdVehicle');/*Valor d'Id No existeix*/
      END;
    END IF;

    IF pIdExt IS NULL THEN
      RETURN RETORNS(2, 0, 'VEHICLES_EXTRES.IdExtra');/*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT Id
          INTO vAux
          FROM EXTRES
         WHERE Id = pIdExt;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdExt, 'VEHICLES_EXTRES.IdExtra');/*Valor d'Id No existeix*/
      END;
    END IF;

    /*Te que se possible fer la relacio, ha de ser en Extres_models o Extres_configfin*/
    BEGIN
      SELECT 1
      INTO vAux
      FROM VEHICLES A, EXTRES_CONFIGFIN B
      WHERE A.Id = pIdVeh AND
            B.IdConFigFin = A.IdConfigFin AND
            B.IdExtra = pIdExt;

    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        BEGIN
          SELECT 1
          INTO vAux
          FROM VEHICLES A, CONFIGFIN B, CONFIG3 C, CONFIG2 D, CONFIG1 E, EXTRES_MODELS F
          WHERE A.Id = pIdVeh AND
                B.Id = A.IdConfigFin AND
                C.Id = B.IdConfig3 AND
                D.Id = C.IdConfig2 AND
                E.Id = D.IdConfig1 AND
                F.IdModel = E.IdModel AND
                F.IdExtra = pIdExt;
        EXCEPTION
          WHEN NO_DATA_FOUND THEN
            RETURN RETORNS (17, pIdVeh, 'VEHICLES_EXTRES.IdVehicle');
        END;
    END;

    BEGIN
      SELECT 1
        INTO vAux
        FROM VEHICLES_EXTRES
       WHERE IdVehicle = pIdVeh AND
             IdExtra = pIdExt;
      IF vAux = 1 THEN
        RETURN RETORNS(14, 0, 'VEHICLES_EXTRES');
      END IF;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
       null;
    END;

    INSERT INTO VEHICLES_EXTRES VALUES (SEQ_VEHICLES_EXTRES.NEXTVAL, pIdVeh, pIdExt)
    RETURNING Id INTO VRes;

    RETURN RETORNS(0, vRes, null);

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VEHICLES_EXTRES', vRes, SQLERRM, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;


  FUNCTION VEHICLES_IMATGES_I (pIdVeh NUMBER, pImatgeG VARCHAR2, pImatgeP VARCHAR2, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vAux NUMBER;
  BEGIN

    IF FUNC_ACCES_PERMES ('VEHICLES_IMATGES_I', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    IF pImatgeG IS NULL THEN
      RETURN RETORNS(2, 0, 'VEHICLES_IMATGES.ImatgeG');/*Valor Null no possible*/
    END IF;

   IF pIdVeh IS NULL THEN
     RETURN RETORNS(2, 0, 'VEHICLES_IMATGES.IdConfigFin');/*Valor Null no possible*/
   ELSE
     BEGIN
       SELECT Id
         INTO vAux
         FROM CONFIGFIN
        WHERE Id = pIdveh;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
         RETURN RETORNS(3, pIdVeh, 'VEHICLES_IMATGES.IdConfigFin');/*Valor d'Id No existeix*/
     END;
   END IF;

    INSERT INTO VEHICLES_IMATGES VALUES (SEQ_VEHICLES_IMATGES.NEXTVAL, pIdVeh, pImatgeG, pImatgeP)
    RETURNING Id INTO VRes;

    RETURN RETORNS(0, vRes, null);

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VEHICLES_IMATGES', vRes, SQLERRM, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION VEHICLES_IMATGES_IDIOMA_I (pIdVehIma NUMBER, pIdId NUMBER, pNom VARCHAR2, pDesc VARCHAR2, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vAux NUMBER;
  BEGIN

    IF FUNC_ACCES_PERMES ('VEHICLES_IMATGES_IDIOMA_I', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    IF pNom IS NULL THEN
      RETURN RETORNS(2, 0, 'VEHICLES_IMATGES_IDIOMA.Nom');/*Valor Null no possible*/
    END IF;

    IF pIdVehIma IS NULL THEN
      RETURN RETORNS(2, 0, 'VEHICLES_IMATGES_IDIOMA.IdVehicleImatge');/*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT Id
          INTO vAux
          FROM VEHICLES_IMATGES
         WHERE Id = pIdVehIma;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3, pIdVehIma, 'VEHICLES_IMATGES_IDIOMA.IdVehicleImatge');/*Valor d'Id No existeix*/
      END;
    END IF;

    IF pIdId IS NULL THEN
      RETURN RETORNS(2, 0, 'VEHICLES_IMATGES_IDIOMA.IdIdioma');/*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT Id
          INTO vAux
          FROM IDIOMES
         WHERE Id = pIdId;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdId, 'VEHICLES_IMATGES_IDIOMA.IdIdioma');/*Valor d'Id No existeix*/
      END;
    END IF;

    INSERT INTO VEHICLES_IMATGES_IDIOMA VALUES (SEQ_VEHICLES_IMATGES_IDIOMA.NEXTVAL, pIdVehIma, pIdId, pNom, pDesc)
    RETURNING Id INTO VRes;

    RETURN RETORNS(0, vRes, null);

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VEHICLES_IMATGES_IDIOMA', vRes, SQLERRM, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION VEHICLES_U (pId NUMBER, pIdCon NUMBER, pIdCli NUMBER, pBastidor VARCHAR2, pMatricula VARCHAR2,
                       pKm NUMBER, pDataRevisio DATE, pDataBaixa DATE, pAnyFabricacio NUMBER, pIdUser NUMBER) RETURN RETORNS  IS
    vAux VARCHAR2 (100);
    vModif BOOLEAN := FALSE;
    vFila VEHICLES%ROWTYPE;
    vSQL VARCHAR2(1000):='';
  BEGIN

    IF FUNC_ACCES_PERMES ('VEHICLES_U', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    IF pAnyFabricacio IS NOT NULL AND (pAnyFabricacio > 2100 OR pAnyFabricacio < 1900)
               THEN RETURN RETORNS(8, 0, 'VEHICLES.AnyFabricacio');/*Any Incorrecte*/
    END IF;

    IF pId IS NULL THEN
      RETURN RETORNS(2, 0, 'VEHICLES.Id');/*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT *
          INTO vFila
          FROM VEHICLES
         WHERE Id = pId;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pId, 'VEHICLES.Id');/*Valor d'Id No existeix*/
      END;
    END IF;

    IF pIdCon IS NOT NULL AND pIdCon <> vFila.IdConfigFin THEN
      BEGIN
        SELECT Id
          INTO vAux
          FROM CONFIGFIN
         WHERE Id = pIdCon;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3, pIdCon, 'VEHICLES.IdConfigFin');/*Valor d'Id No existeix*/
      END;
      vSQL := vSQL||' IdConfigFin = ' || pIdCon||',';
      vModif := TRUE;
    END IF;

    IF pIdCli IS NOT NULL AND pIdCli <> vFila.IdClient THEN
      BEGIN
        SELECT Id
          INTO vAux
          FROM CLIENTS
         WHERE Id = pIdCli;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3, pIdCli, 'VEHICLES.IdClient');/*Valor d'Id No existeix*/
      END;
      vSQL := vSQL||' IdClient = ' || pIdCli||',';
      vModif := TRUE;
    END IF;

    IF pBastidor IS NOT NULL AND pBastidor <> vFila.Bastidor THEN
      vSQL := vSQL||' Tipus = ''' || pBastidor ||''',';
      vModif := TRUE;
    END IF;

    IF pMatricula IS NOT NULL AND pMatricula <> vFila.Matricula THEN
      vSQL := vSQL||' Matricula = ''' || pMatricula ||''',';
      vModif := TRUE;
    END IF;

    IF pKm IS NOT NULL AND pKm <> vFila.Km THEN
      vSQL := vSQL||' Km = ' || pKm ||',';
      vModif := TRUE;
    END IF;

    IF pDataRevisio IS NOT NULL AND pDataRevisio <> vFila.DataRevisio THEN
      vSQL := vSQL||' DataRevisio = ''' || pDataRevisio ||''',';
      vModif := TRUE;
    END IF;

    IF pDataBaixa IS NOT NULL AND pDataBaixa <> vFila.DataBaixa THEN
      vSQL := vSQL||' DataBaixa = ''' || pDataBaixa ||''',';
      vModif := TRUE;
    END IF;

    IF pAnyFabricacio IS NOT NULL AND pAnyFabricacio <> vFila.AnyFabricacio THEN
      vSQL := vSQL||' AnyFabricacio = ' || pAnyFabricacio ||',';
      vModif := TRUE;
    END IF;

    IF vModif THEN
      vSQL := 'UPDATE VEHICLES SET ' || SUBSTR(vSQL, 1,LENGTH (vSQL)-1)|| ' WHERE Id = '|| pId;
      EXECUTE IMMEDIATE (vSQL);

      IF sql%ROWCOUNT >=1 THEN
        RETURN RETORNS(0, pId, null);
      ELSE
        RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
      END IF;
    ELSE
      RETURN RETORNS(4, pId, null);/*No s'ha realitzat cap acci*/
    END IF;
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VEHICLES', pId, vSQL, 'U', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION VEHICLES_EXTRES_U (pId NUMBER, pIdVeh NUMBER, pIdExt NUMBER, pIdUser NUMBER) RETURN RETORNS IS

  BEGIN

    IF FUNC_ACCES_PERMES ('VEHICLES_EXTRES_U', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    RETURN RETORNS (15, 0, 'VEHICLES_EXTRES_U');

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VEHICLES_EXTRES', pId, SQLERRM, 'U', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION VEHICLES_IMATGES_U (pId NUMBER, pIdVeh NUMBER, pImatgeG VARCHAR2, pImatgeP VARCHAR2, pIdUser NUMBER) RETURN RETORNS IS

    vAux VARCHAR2 (100);
    vModif BOOLEAN := FALSE;
    vFila VEHICLES_IMATGES%ROWTYPE;
    vSQL VARCHAR2(1000):='';
  BEGIN

    IF FUNC_ACCES_PERMES ('VEHICLES_IMATGES_U', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    IF pId IS NULL THEN
      RETURN RETORNS(2, 0, 'VEHICLES_IMATGES.Id');/*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT *
          INTO vFila
          FROM VEHICLES_IMATGES
         WHERE Id = pId;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pId, 'VEHICLES_IMATGES.Id');/*Valor d'Id No existeix*/
      END;
    END IF;

    IF pIdVeh IS NOT NULL AND pIdVeh <> vFila.IdVehicle THEN
      BEGIN
        SELECT Id
          INTO vAux
          FROM VEHICLES
         WHERE Id = pIdVeh;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdVeh, 'VEHICLES_IMATGES.IdModel');/*Valor d'Id No existeix*/
      END;
      vSQL := vSQL||' IdVehicle = ' || pIdVeh||',';
      vModif := TRUE;
    END IF;

    IF pImatgeG <> vFila.ImatgeG THEN
      vSQL := vSQL||' ImatgeG = ''' || pImatgeG||''',';
      vModif := TRUE;
    END IF;

    IF pImatgeP <> vFila.ImatgeP THEN
      vSQL := vSQL||' ImatgeP = ''' || pImatgeP||''',';
      vModif := TRUE;
    END IF;

    IF vModif THEN
      vSQL := 'UPDATE VEHICLES_IMATGES SET ' || SUBSTR(vSQL, 1,LENGTH (vSQL)-1)|| ' WHERE Id = '|| pId;
      EXECUTE IMMEDIATE (vSQL);

      IF sql%ROWCOUNT >=1 THEN
        RETURN RETORNS(0, pId, null);
      ELSE
        RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
      END IF;
    ELSE
      RETURN RETORNS(4, pId, null);/*No s'ha realitzat cap acci*/
    END IF;
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VEHICLES_IMATGES', pId, vSQL, 'U', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION VEHICLES_IMATGES_IDIOMA_U (pId NUMBER, pIdVehIma NUMBER, PIdId NUMBER, pNom VARCHAR2, pDesc VARCHAR2, pIdUser NUMBER) RETURN RETORNS IS

    vAux VARCHAR2 (100);
    vModif BOOLEAN := FALSE;
    vFila VEHICLES_IMATGES_IDIOMA%ROWTYPE;
    vSQL VARCHAR2(1000):='';
  BEGIN

    IF FUNC_ACCES_PERMES ('VEHICLES_IMATGES_IDIOMA_U', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

   IF pId IS NULL THEN
     RETURN RETORNS(2, 0, 'VEHICLES_IMATGES_IDIOMA.Id');/*Valor Null no possible*/
   ELSE
     BEGIN
       SELECT *
         INTO vFila
         FROM VEHICLES_IMATGES_IDIOMA
        WHERE Id = pId;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
         RETURN RETORNS(3,pId, 'VEHICLES_IMATGES_IDIOMA.Id');/*Valor d'Id No existeix*/
     END;
   END IF;

    IF pIdVehIma IS NOT NULL AND pIdVehIma <> vFila.IdVehicleImatge THEN
      BEGIN
        SELECT Id
          INTO vAux
          FROM CONFIGFIN_IMATGES
         WHERE Id = pIdVehIma;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdVehIma, 'VEHICLES_IMATGES_IDIOMA.IdVehicleImatge');/*Valor d'Id No existeix*/
      END;
      vSQL := vSQL||' IdVehicleImatge = ' || pIdVehIma||',';
      vModif := TRUE;
    END IF;

    IF pDesc IS NOT NULL AND pDesc <> vFila.Descripcio THEN
      vSQL := vSQL||' Descripcio = ''' || pDesc ||''',';
      vModif := TRUE;
    END IF;

    IF pIdId IS NOT NULL AND pIdId <> vFila.IdIdioma THEN
      BEGIN
        SELECT Id
          INTO vAux
          FROM IDIOMES
         WHERE Id = pIdId;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdId, 'VEHICLES_IMATGES_IDIOMA.IdIdioma');/*Valor d'Id No existeix*/
      END;
      vSQL := vSQL||' IdIdioma = ' || pIdId||',';
      vModif := TRUE;
    END IF;

    IF pNom <> vFila.Nom THEN
      vSQL := vSQL||' Nom = ''' || pNom||''',';
      vModif := TRUE;
    END IF;

    IF vModif THEN
      vSQL := 'UPDATE VEHICLES_IMATGES_IDIOMA SET ' || SUBSTR(vSQL, 1,LENGTH (vSQL)-1)|| ' WHERE Id = '|| pId;
      EXECUTE IMMEDIATE (vSQL);

      IF sql%ROWCOUNT >=1 THEN
        RETURN RETORNS(0, pId, null);
      ELSE
        RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
      END IF;
    ELSE
      RETURN RETORNS(4, pId, null);/*No s'ha realitzat cap acci*/
    END IF;
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VEHICLES_IMATGES_IDIOMA', pId, vSQL, 'U', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;


FUNCTION VEHICLES_D (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
  BEGIN

    IF FUNC_ACCES_PERMES ('VEHICLES_D', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    BEGIN
      SELECT Id
        INTO vAux
        FROM VEHICLES
       WHERE Id = pId;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pId, 'VEHICLES.Id');/*Valor d'Id No existeix*/
    END;

    DELETE VEHICLES WHERE Id = pId;

    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha esborrat registre*/
    END IF;

  EXCEPTION
    WHEN TE_FILLS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VEHICLES', pId, 'FK Error: '||SQLERRM, 'D', 0);
      RETURN RETORNS(6,pId, 'VEHICLES.Id');/*Error general revisar LOG*/
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VEHICLES', pId, SQLERRM, 'D', 0);
      RETURN RETORNS(1,pId, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION VEHICLES_EXTRES_D (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
  BEGIN

    IF FUNC_ACCES_PERMES ('VEHICLES_EXTRES_D', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    IF pId IS NULL THEN
      RETURN RETORNS(2, 0, 'VEHICLES_EXTRES.Id');/*Valor Null no possible*/
    END IF;

    BEGIN
      SELECT Id
        INTO vAux
        FROM VEHICLES_EXTRES
       WHERE Id = pId;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pId, 'VEHICLES_EXTRES.pId');/*Valor d'Id No existeix*/
    END;

    DELETE VEHICLES_EXTRES WHERE Id = pId;

    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha esborrat registre*/
    END IF;

  EXCEPTION
   WHEN TE_FILLS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VEHICLES_EXTRES', pId, 'FK Error: '||SQLERRM, 'D', 0);
      RETURN RETORNS(6,pId, 'VEHICLES_EXTRES.Id');/*Error general revisar LOG*/
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VEHICLES_EXTRES', pId, SQLERRM, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION VEHICLES_IMATGES_D (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
  BEGIN

    IF FUNC_ACCES_PERMES ('VEHICLES_IMATGES_D', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    IF pId IS NULL THEN
      RETURN RETORNS(2, 0, 'VEHICLES_IMATGES.Id');/*Valor Null no possible*/
    END IF;

    BEGIN
      SELECT Id
        INTO vAux
        FROM VEHICLES_IMATGES
       WHERE Id = pId;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pId, null);/*Valor d'Id No existeix*/
    END;

    DELETE VEHICLES_IMATGES WHERE Id = pId;

    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha esborrat registre*/
    END IF;

  EXCEPTION
   WHEN TE_FILLS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VEHICLES_IMATGES', pId, 'FK Error: '||SQLERRM, 'D', 0);
      RETURN RETORNS(6,pId, 'VEHICLES_IMATGES.Id');/*Error general revisar LOG*/
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VEHICLES_IMATGES', pId, SQLERRM, 'D', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION VEHICLES_IMATGES_IDIOMA_D (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
  BEGIN

    IF FUNC_ACCES_PERMES ('VEHICLES_IMATGES_IDIOMA_D', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;

    IF pId IS NULL THEN
      RETURN RETORNS(2, 0, 'VEHICLES_IMATGES_IDIOMA.Id');/*Valor Null no possible*/
    END IF;

    BEGIN
      SELECT Id
        INTO vAux
        FROM VEHICLES_IMATGES_IDIOMA
       WHERE Id = pId;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pId, null);/*Valor d'Id No existeix*/
    END;

    DELETE VEHICLES_IMATGES_IDIOMA WHERE Id = pId;

    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha esborrat registre*/
    END IF;

  EXCEPTION
   WHEN TE_FILLS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VEHICLES_IMATGES_IDIOMA', pId, 'FK Error: '||SQLERRM, 'D', 0);
      RETURN RETORNS(6,pId, 'VEHICLES_IMATGES_IDIOMA.Id');/*Error general revisar LOG*/
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VEHICLES_IMATGES_IDIOMA', pId, SQLERRM, 'D', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;
  
  FUNCTION CONSULTA_VEHICLES_PER_REVISIO (pVehiclesTab OUT VEHICLES_TAB, pIdUser NUMBER) RETURN RETORNS IS
  
    CURSOR cVehicles IS
    SELECT *
    FROM VEHICLES
    WHERE Datarevisio BETWEEN SYSDATE-30 AND SYSDATE+30;
    
    vVehicles VEHICLES%ROWTYPE;
    i NUMBER :=1;
  
  BEGIN
  
    IF FUNC_ACCES_PERMES ('CONSULTA_VEHICLES_PER_REVISIO', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF;
       
    OPEN cVehicles;
      FETCH cVehicles INTO vVehicles;
      pVehiclesTab  := VEHICLES_TAB();
      WHILE cVehicles%found LOOP
      pVehiclesTab.EXTEND;
      pVehiclesTab(i) := VEHICLES_TYP(vVehicles.Id, vVehicles.IdConfigFin, vVehicles.IdClient,
                                      vVehicles.Bastidor, vVehicles.Matricula, vVehicles.Km,
                                      vVehicles.DataRevisio, vVehicles.DataBaixa, vVehicles.AnyFabricacio);
      FETCH cVehicles INTO vVehicles;
        i := i + 1;
      END LOOP;
    CLOSE cVehicles;

    RETURN RETORNS(0,0, 'CONSULTA_VEHICLES_PER_REVISIO');
    
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'CONSULTA_VEHICLES_PER_REVISIO', 0, SQLERRM, 'D', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

END PAK_VEHICLES;