create or replace
PACKAGE BODY PAK_VENTES AS

  TE_FILLS exception;
  pragma exception_init( TE_FILLS, -2292 );
  
  FUNCTION ENCARREGAR_VEHICLE (pIdVehicle NUMBER, pIdClient NUMBER, pIdCentre NUMBER, pIdTreballador NUMBER,  
                     pPagat NUMBER, pDescompte NUMBER, pComentari VARCHAR2, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vAux VARCHAR2 (100);
    vAuxV VARCHAR2 (100);
    vFila STOCK%ROWTYPE; 
    vPreuFinal NUMBER;
    vRet RETORNS;
  BEGIN 

    IF FUNC_ACCES_PERMES ('ENCARREGAR_VEHICLE', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
    
    IF pIdVehicle IS NULL THEN 
      RETURN RETORNS(2, 0, 'VENTES.IdVehicle');/*Valor Null no possible*/
    ELSE     
      BEGIN
        SELECT IdConfigfin
          INTO vAuxV
          FROM VEHICLES
         WHERE Id = pIdVehicle;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdVehicle, 'VENTES.IdVehicle');/*Valor d'Id No existeix*/
      END;
    END IF;
    
    IF pIdCentre IS NULL THEN 
      RETURN RETORNS(2, 0, 'VENTES.IdCentre'); /*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT Id
          INTO vAux
          FROM CENTRES
         WHERE Id = pIdCentre;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdCentre, 'VENTES.IdCentre');/*Valor d'Id No existeix*/
      END;
    END IF;
    
    IF pIdClient IS NULL THEN 
      RETURN RETORNS(2, 0, 'VENTES.IdClient'); /*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT Id
          INTO vAux
          FROM CLIENTS
         WHERE Id = pIdClient;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdClient, 'VENTES.IdClient');/*Valor d'Id No existeix*/
      END;
    END IF;
 
    IF pIdTreballador IS NULL THEN 
      RETURN RETORNS(2, 0, 'VENTES.pIdTreballador'); /*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT Id
          INTO vAux
          FROM TREBALLADORS
         WHERE Id = pIdTreballador;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdTreballador, 'VENTES.IdTreballador');/*Valor d'Id No existeix*/
      END;
    END IF;
    BEGIN
      SELECT *
        INTO vFila
        FROM STOCK
       WHERE IdVehicle = vFila.IdVehicle AND
             IdCentre = vFila.IdCentre AND
             DataSortida IS NULL;
    vRet := PAK_STOCK.RESERVA_VEHICLE(pIdVehicle, pIdCentre,0);             
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        BEGIN
          SELECT Preu
            INTO vFila.PreuVenta
            FROM CONFIGFIN
           WHERE Id = vAuxV;
          vFila.Us := 'N';
       EXCEPTION
         WHEN NO_DATA_FOUND THEN
           RETURN  RETORNS(3,vAuxV, 'VEHICLES.IdConfigFin');/*Valor d'Id No existeix*/
       END;
    END;    
    IF pDescompte > 0 THEN
      vPreuFinal := vFila.PreuVenta -pDescompte;
    ELSE
      vPreuFinal := vFila.PreuVenta;
    END IF;
    
    INSERT INTO VENTES (Id, IdVehicle, IdClient, IdCentre, IdTreballador, 
                        DataEncarrec, Pagat, Us, Preu, Descompte, PreuFinal, Comentari)
                VALUES (SEQ_VENTES.NEXTVAL, pIdVehicle, pIdClient, pIdCentre, pIdTreballador, 
                        SYSDATE, pPagat, vFila.Us, vFila.PreuVenta, pDescompte,vPreuFinal, pComentari) 
    RETURNING Id INTO vRes;

    RETURN RETORNS(0, vRes, null);
  
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'COMPRES', vRes, SQLERRM, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION VENTES_U (pId NUMBER, pDescompte NUMBER, pDataPrevEntrega DATE,  
                   pComentari VARCHAR2, pIdUser NUMBER) RETURN RETORNS  IS
    vAux VARCHAR2 (100);
    vModif BOOLEAN := FALSE;
    vFila VENTES%ROWTYPE;
    vSQL VARCHAR2(1000):='';
    BEGIN 

    IF FUNC_ACCES_PERMES ('VENTES_U', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
    
    IF pId IS NULL THEN 
      RETURN RETORNS(2, 0, 'VENTES.Id');/*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT *
          INTO vFila
          FROM VENTES
         WHERE Id = pId;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pId, 'VENTES.Id');/*Valor d'Id No existeix*/
      END;
    END IF;

    IF pDescompte IS NOT NULL AND pDescompte <> vFila.Descompte THEN
      vSQL := vSQL||' Descompte = ' || pDescompte ||',';
      vModif := TRUE;
    END IF;

    IF pDataPrevEntrega IS NOT NULL AND pDataPrevEntrega <> vFila.DataPrevEntrega THEN
      vSQL := vSQL||' DataPrevEntrega = ''' || pDataPrevEntrega ||''',';
      vModif := TRUE;
    END IF;

    IF pComentari IS NOT NULL AND pComentari <> vFila.Comentari THEN
      vSQL := vSQL||' Comentari = ''' || pComentari ||''',';
      vModif := TRUE;
    END IF;
  
    IF vModif THEN
      vSQL := 'UPDATE VENTES SET ' || SUBSTR(vSQL, 1,LENGTH (vSQL)-1)|| ' WHERE Id = '|| pId;
      EXECUTE IMMEDIATE (vSQL);

      IF sql%ROWCOUNT >=1 THEN
        RETURN RETORNS(0, pId, null);
      ELSE
        RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
      END IF;
    ELSE
      RETURN RETORNS(4, pId, null);/*No s'ha realitzat cap acci*/
    END IF;
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VENTES', pId, vSQL, 'U', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;
   
  FUNCTION VENTES_D (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
    vFila VENTES%ROWTYPE;
    vRet RETORNS;
  BEGIN 
 
    IF FUNC_ACCES_PERMES ('VENTES_D', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
      
    IF pId IS NULL THEN 
      RETURN RETORNS(2, 0, 'VENTES.Id');/*Valor Null no possible*/ 
    ELSE
      BEGIN
        SELECT *
          INTO vFila
          FROM VENTES
         WHERE Id = pId;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pId, 'VENTES.Id');/*Valor d'Id No existeix*/
      END;
    END IF;
    
    IF vFila.DataEntrega IS NOT NULL THEN
      RETURN RETORNS(46,pId, 'VENTES.Id');/*Entregada*/
    END IF;
    /* es comproba data de tancament, per que si es recepcionada ha de ser tancada*/
    IF vFila.Pagat <> 0 THEN
      RETURN RETORNS(45,pId, 'VENTES.Id');/*amb paga i senyal*/
    END IF;
        
    DELETE VENTES WHERE Id = pId;
    
    
    IF sql%ROWCOUNT >=1 THEN
      BEGIN
        SELECT Id
          INTO vAux
          FROM STOCK
         WHERE Idvehicle = vFila.IdVehicle;
       vRet := PAK_STOCK.DESRESERVA_VEHICLE(vFila.IdVehicle, vFila.IdCentre,0);
      RETURN vRet;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(0,pId, null);/*Valor d'Id No existeix*/
      END;

    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha esborrat registre*/
    END IF;
  
  EXCEPTION
    WHEN TE_FILLS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VENTES', pId, 'FK Error: '||SQLERRM, 'D', 0);
      RETURN RETORNS(6,pId, 'VENTES.Id');/*Error general revisar LOG*/
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VENTES', pId, SQLERRM, 'D', 0);
      RETURN RETORNS(1,pId, SQLCODE);/*Error general revisar LOG*/
  END;
 
   FUNCTION BAIXA_VENTA (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
    vFila VENTES%ROWTYPE;
    vRet RETORNS;
  BEGIN 
 
    IF FUNC_ACCES_PERMES ('BAIXA_VENTA', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
      
    IF pId IS NOT NULL THEN 
      RETURN RETORNS(2, 0, 'VENTES.Id');/*Valor Null no possible*/ 
    ELSE
      BEGIN
        SELECT *
          INTO vFila
          FROM VENTES
         WHERE Id = pId;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pId, 'VENTES.Id');/*Valor d'Id No existeix*/
      END;
    END IF;
    
    IF vFila.DataBaixa IS NOT NULL THEN
      RETURN RETORNS(47,pId, 'VENTES.Id');/*ja de baixa*/
    END IF;

    IF vFila.DataEntrega IS NOT NULL THEN
      RETURN RETORNS(48,pId, 'VENTES.Id');/*ja entregada*/
    END IF;
    
    UPDATE VENTES SET DataBaixa = SYSDATE WHERE Id = pId;
    
    IF sql%ROWCOUNT >=1 THEN
    vRet := PAK_STOCK.DESRESERVA_VEHICLE(vFila.IdVehicle, vFila.IdCentre,0);
    RETURN vRet;
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha esborrat registre*/
    END IF;
  
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'BAIXA_VENTA', pId, SQLERRM, 'F', 0);
      RETURN RETORNS(1,pId, SQLCODE);/*Error general revisar LOG*/
  END;
     
  FUNCTION ENTREGA_VENTA (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vFila VENTES%ROWTYPE;
    vFilaVeh VEHICLES%ROWTYPE;
    vAux VARCHAR2 (100);
    vRet RETORNS;
  BEGIN     
 
    IF FUNC_ACCES_PERMES ('ENTREGA_VENTA', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
   
    IF pId IS NOT NULL THEN 
      RETURN RETORNS(2, 0, 'VENTES.Id');/*Valor Null no possible*/ 
    ELSE
      BEGIN
        SELECT *
          INTO vFila
          FROM VENTES
         WHERE Id = pId;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pId, 'VENTES.Id');/*Valor d'Id No existeix*/
      END;
    END IF;

    BEGIN
      SELECT Id
        INTO vAux
        FROM STOCK
       WHERE IdVehicle = vFila.IdVehicle AND
             IdCentre = vFila.IdCentre AND
             DataSortida IS NULL;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(44,vFila.IdVehicle, 'VENTES.IdVehicle');/*Valor d'Id No existeix*/
      END;

    IF vFila.DataBaixa IS NOT NULL THEN
      RETURN RETORNS(49,pId, 'COMPRES.Id');/*donada de baixa*/
    END IF;
    
    IF vFila.Pagat < vFila.preuFinal THEN
      RETURN RETORNS(51,pId, 'VENTES.Id');/*donada de baixa*/
    END IF;
    BEGIN
      SELECT *
      INTO vFilaVeh
      FROM VEHICLES 
      WHERE Id = vFila.IdVehicle;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pId, 'VENTES.Id');/*Valor d'Id No existeix*/
    END;
    IF vFilaVeh.Matricula IS NULL THEN
      RETURN RETORNS(53,pId, 'VENTES.Id');/*donada de baixa*/
    END IF;
    
    IF vFilaVeh.Bastidor IS NULL THEN
      RETURN RETORNS(54,pId, 'VENTES.Id');/*donada de baixa*/
    END IF;
    
    UPDATE VENTES SET DataEntrega = SYSDATE WHERE Id = pId;
    
    IF sql%ROWCOUNT >=1 THEN
      vRet := PAK_STOCK.TREU_DE_STOCK(vFila.IdVehicle, vFila.IdCentre,0);
    RETURN vRet;
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
    END IF;

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'ENTREGA_VENTA', vRes, SQLERRM, 'F', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION TANCAR_VENTA (pIdVenta NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vFila VENTES%ROWTYPE;
    vAux VARCHAR2 (100);
      
  BEGIN     
 
    IF FUNC_ACCES_PERMES ('TANCAR_VENTA', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
   
    IF pIdVenta IS NOT NULL THEN 
      RETURN RETORNS(2, 0, 'VENTES.Id');/*Valor Null no possible*/ 
    ELSE
      BEGIN
        SELECT *
          INTO vFila
          FROM VENTES
         WHERE Id = pIdVenta;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(55,pIdVenta, 'VENTES.Id');/*Valor d'Id No existeix*/
      END;
    END IF;
    
    IF vFila.DataBaixa IS NOT NULL THEN
      RETURN RETORNS(49,pIdVenta, 'VENTES.Id');/*donada de baixa*/
    END IF;
    
    IF vFila.Pagat IS NULL OR vFila.Pagat > 0 THEN
      RETURN RETORNS(52,pIdVenta, 'VENTES.Id');/*donada de baixa*/
    END IF;

    UPDATE VENTES SET DataVenta = SYSDATE WHERE Id = pIdVenta;
    
    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pIdVenta, null);
    ELSE
      RETURN RETORNS(5, pIdVenta, null); /*No s'ha modificat registre*/
    END IF;

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'ENTREGA_VENTA', pIdVenta, SQLERRM, 'F', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;
  
  FUNCTION AMPLIAR_PAGAMENT (pIdVenta NUMBER, pPagatASumar NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vFila VENTES%ROWTYPE;
    vAux VARCHAR2 (100);
      
  BEGIN     
 
    IF FUNC_ACCES_PERMES ('AMPLIAR_PAGAMENT', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
    
    IF pPagatASumar IS NOT NULL THEN 
      RETURN RETORNS(2, 0, 'VENTES.PagatASumar');/*Valor Null no possible*/ 
    END IF;
    
    IF pIdVenta IS NOT NULL THEN 
      RETURN RETORNS(2, 0, 'VENTES.Id');/*Valor Null no possible*/ 
    ELSE
      BEGIN
        SELECT *
          INTO vFila
          FROM VENTES
         WHERE Id = pIdVenta;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdVenta, 'VENTES.Id');/*Valor d'Id No existeix*/
      END;
    END IF;
    
    IF vFila.DataBaixa IS NOT NULL THEN
      RETURN RETORNS(49,pIdVenta, 'VENTES.Id');/*donada de baixa*/
    END IF;
    
    IF vFila.Pagat + pPagatASumar > vFila.PreuFinal THEN
      RETURN RETORNS(50,pIdVenta, 'VENTES.Id');
    END IF;
    
    UPDATE VENTES SET Pagat = vFila.Pagat + pPagatASumar WHERE Id = pIdVenta;
    
    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pIdVenta, null);
    ELSE
      RETURN RETORNS(5, pIdVenta, null); /*No s'ha modificat registre*/
    END IF;

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'ENTREGA_VENTA', pIdVenta, SQLERRM, 'F', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;
  
END PAK_VENTES;
/