create or replace
PACKAGE BODY PAK_VERSIO AS

  TE_FILLS exception;
  pragma exception_init( TE_FILLS, -2292 );

  FUNCTION VERSIO_I (pNom VARCHAR2, pAnyVersio NUMBER, pCambi VARCHAR2, pPortes NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
  BEGIN 

    IF FUNC_ACCES_PERMES ('VERSIO_I', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
    
    IF pNom IS NULL THEN RETURN RETORNS(2, 0, 'VERSIO.Nom');/*Valor Null no possible*/
    ELSIF pAnyVersio IS NOT NULL AND (pAnyVersio > 2100 OR pAnyVersio < 1900) THEN 
                                          RETURN RETORNS(8, 0, 'VERSIO.AnyVersio');/*Any Incorrecte*/
    ELSIF pCambi IS NULL THEN RETURN RETORNS(2, 0, 'VERSIO.Cambi');/*Valor Null no possible*/
    ELSIF pCambi NOT IN ('M', 'A') THEN RETURN RETORNS(13, 0, 'VERSIO.Cambi');/*Cambi Incorrecte*/               
    END IF;

    INSERT INTO VERSIO (Id, Nom, AnyVersio, Cambi, Portes)
                VALUES (SEQ_VERSIO.NEXTVAL, pNom, pAnyVersio, pCambi, pPortes)
    RETURNING Id INTO vRes;

    RETURN RETORNS(0, vRes,null);
 
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VERSIO', vRes, pNom, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION VERSIO_IDIOMA_I (pIdVer NUMBER, pIdId NUMBER, pNom VARCHAR2, pDesc VARCHAR2, pIdUser NUMBER) RETURN RETORNS IS
    vRes NUMBER;
    vAux NUMBER;
  BEGIN 

    IF FUNC_ACCES_PERMES ('VERSIO_IDIOMA_I', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
    
    IF pIdVer IS NULL THEN RETURN RETORNS(2, 0, 'VERSIO_IDIOMA.IdVersio');/*Valor Null no possible*/
    ELSIF pIdId IS NULL THEN RETURN RETORNS(2, 0, 'VERSIO_IDIOMA.IdIdioma');/*Valor Null no possible*/
    ELSIF pNom IS NULL THEN RETURN RETORNS(2, 0, 'VERSIO_IDIOMA.Nom');/*Valor Null no possible*/
    END IF;

    BEGIN
      SELECT Id
        INTO vAux
        FROM IDIOMES
       WHERE Id = pIdId;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3, pIdId, 'VERSIO_IDIOMA.IdIdioma');/*Valor d'Id No existeix*/
    END;

    BEGIN
      SELECT Id
        INTO vAux
        FROM VERSIO
       WHERE Id = pIdVer;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pIdVer, 'VERSIO_IDIOMA.IdVersio');/*Valor d'Id No existeix*/
    END;

    INSERT INTO VERSIO_IDIOMA VALUES (SEQ_VERSIO_IDIOMA.NEXTVAL, pIdVer, pIdId, pNom, pDesc)
    RETURNING Id INTO VRes;

    RETURN RETORNS(0, vRes, null);

  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VERSIO_IDIOMA', vRes, pNom, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION VERSIO_U (pId NUMBER, pNom VARCHAR2, pAnyVersio NUMBER, pCambi VARCHAR2, pPortes NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vModif BOOLEAN := FALSE;
    vFila VERSIO%ROWTYPE;
    vSQL VARCHAR2(1000):='';
  BEGIN 

    IF FUNC_ACCES_PERMES ('VERSIO_U', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
    
    IF pAnyVersio IS NOT NULL AND (pAnyVersio > 2100 OR pAnyVersio < 1900) THEN 
      RETURN RETORNS(8, 0, 'VERSIO.AnyVersio');/*Any Incorrecte*/
    END IF;
    IF pCambi IS NOT NULL AND pCambi NOT IN ('M', 'A') THEN 
      RETURN RETORNS(13, 0, 'VERSIO.Cambi');/*Cambi Incorrecte*/    
    END IF;
    
    IF pId IS NULL 
      THEN RETURN RETORNS(2, 0, 'VERSIO.Id');/*Valor Null no possible*/
    ELSE
      BEGIN
        SELECT *
          INTO vFila
          FROM VERSIO
         WHERE Id = pId;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pId, 'VERSIO.Id'); /*Valor d'Id No existeix*/
      END;
    END IF;

    IF pNom IS NOT NULL AND pNom <> vFila.Nom THEN
      vSQL := vSQL||' Nom = ''' || pNom ||''',';
      vModif := TRUE;
    END IF;

   IF pAnyVersio IS NOT NULL AND pAnyVersio <> vFila.AnyVersio THEN
      vSQL := vSQL||' AnyVersio = ' || pAnyVersio ||',';
      vModif := TRUE;
    END IF;

   IF pCambi IS NOT NULL AND pCambi <> vFila.Cambi THEN
      vSQL := vSQL||' Cambi = ''' || pCambi ||''',';
      vModif := TRUE;
   END IF;

   IF pPortes IS NOT NULL AND pPortes <> vFila.Portes THEN
      vSQL := vSQL||' Portes = ' || pPortes ||',';
      vModif := TRUE;
    END IF;

    IF vModif THEN
      vSQL := 'UPDATE VERSIO SET ' || SUBSTR(vSQL, 1,LENGTH (vSQL)-1)|| ' WHERE Id = '|| pId;
      EXECUTE IMMEDIATE (vSQL);

      IF sql%ROWCOUNT >=1 THEN
        RETURN RETORNS(0, pId, null);
      ELSE
        RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
      END IF;
    ELSE
      RETURN RETORNS(4, pId, null);/*No s'ha realitzat cap acci*/
    END IF;
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VERSIO', pId, vSQL, 'U', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION VERSIO_IDIOMA_U (pId NUMBER, pIdVer NUMBER, PIdId NUMBER, pNom VARCHAR2, pDesc VARCHAR2, pIdUser NUMBER) RETURN RETORNS IS

    vAux VARCHAR2 (100);
    vModif BOOLEAN := FALSE;
    vFila VERSIO_IDIOMA%ROWTYPE;
    vSQL VARCHAR2(1000):='';
  BEGIN 

    IF FUNC_ACCES_PERMES ('VERSIO_IDIOMA_U', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
    
   IF pId IS NULL THEN 
     RETURN RETORNS(2, 0, 'VERSIO_IDIOMA.Id');/*Valor Null no possible*/
   ELSE
     BEGIN
       SELECT *
         INTO vFila
         FROM VERSIO_IDIOMA
        WHERE Id = pId;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
         RETURN RETORNS(3,pId, 'VERSIO_IDIOMA.Id');/*Valor d'Id No existeix*/
     END;
   END IF;
   
    IF pIdVer IS NOT NULL AND pIdVer <> vFila.IdVersio THEN
      BEGIN
        SELECT Id
          INTO vAux
          FROM VERSIO
         WHERE Id = pIdVer;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdVer, 'VERSIO_IDIOMA.IdVersio');/*Valor d'Id No existeix*/
      END;
      vSQL := vSQL||' IdVersio = ' || pIdVer||',';
      vModif := TRUE;
    END IF;
  
    IF pDesc IS NOT NULL AND pDesc <> vFila.Descripcio THEN
      vSQL := vSQL||' Descripcio = ''' || pDesc ||''',';
      vModif := TRUE;
    END IF;

    IF pIdId IS NOT NULL AND pIdId <> vFila.IdIdioma THEN  
      BEGIN
        SELECT Id
          INTO vAux
          FROM IDIOMES
         WHERE Id = pIdId;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RETURN RETORNS(3,pIdId, 'VERSIO_IDIOMA.IdIdioma');/*Valor d'Id No existeix*/
      END;
      vSQL := vSQL||' IdIdioma = ' || pIdId||',';
      vModif := TRUE;
    END IF;  
  
    IF pNom <> vFila.Nom THEN
      vSQL := vSQL||' Nom = ''' || pNom||''',';
      vModif := TRUE;
    END IF;

    IF vModif THEN
      vSQL := 'UPDATE VERSIO_IDIOMA SET ' || SUBSTR(vSQL, 1,LENGTH (vSQL)-1)|| ' WHERE Id = '|| pId;
      EXECUTE IMMEDIATE (vSQL);

      IF sql%ROWCOUNT >=1 THEN
        RETURN RETORNS(0, pId, null);
      ELSE
        RETURN RETORNS(5, pId, null); /*No s'ha modificat registre*/
      END IF;
    ELSE
      RETURN RETORNS(4, pId, null);/*No s'ha realitzat cap acci*/
    END IF;
  EXCEPTION
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VERSIO_IDIOMA', pId, vSQL, 'U', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

  FUNCTION VERSIO_D (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
  BEGIN 
 
    IF FUNC_ACCES_PERMES ('VERSIO_D', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
     
    BEGIN
      SELECT Id
        INTO vAux
        FROM VERSIO
       WHERE Id = pId;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pId, 'VERSIO.Id');/*Valor d'Id No existeix*/
    END;
    
    DELETE VERSIO WHERE Id = pId;
    
    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha esborrat registre*/
    END IF;
  
  EXCEPTION
    WHEN TE_FILLS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VERSIO', pId, 'FK Error: '||SQLERRM, 'D', 0);
      RETURN RETORNS(6,pId, 'VERSIO.Id');/*Error general revisar LOG*/
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VERSIO', pId, null, 'D', 0);
      RETURN RETORNS(1,pId, SQLCODE);/*Error general revisar LOG*/
  END;
  
  FUNCTION VERSIO_IDIOMA_D (pId NUMBER, pIdUser NUMBER) RETURN RETORNS IS
    vAux VARCHAR2 (100);
  BEGIN 

    IF FUNC_ACCES_PERMES ('VERSIO_IDIOMA_D', pIdUser) = FALSE THEN
      RETURN RETORNS(10, pIdUser, 'TREBALLADORS.IdUser');
    END IF; 
      
    IF pId IS NULL THEN
      RETURN RETORNS(2, 0, 'VERSIO_IDIOMA.Id');/*Valor Null no possible*/
    END IF;
  
    BEGIN
      SELECT Id
        INTO vAux
        FROM VERSIO_IDIOMA
       WHERE Id = pId;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
        RETURN RETORNS(3,pId, null);/*Valor d'Id No existeix*/
    END;
  
    DELETE VERSIO_IDIOMA WHERE Id = pId;
    
    IF sql%ROWCOUNT >=1 THEN
      RETURN RETORNS(0, pId, null);
    ELSE
      RETURN RETORNS(5, pId, null); /*No s'ha esborrat registre*/
    END IF;
  
  EXCEPTION
   WHEN TE_FILLS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VERSIO', pId, 'FK Error: '||SQLERRM, 'D', 0);
      RETURN RETORNS(6,pId, 'VERSIO_IDIOMA.Id');/*Error general revisar LOG*/  
    WHEN OTHERS THEN
      PROC_INSERT_LOGERROR(SQLCODE, 'VERSIO', pId, SQLERRM, 'I', 0);
      RETURN RETORNS(1,0, SQLCODE);/*Error general revisar LOG*/
  END;

END PAK_VERSIO;    
/